#!/usr/bin/python

__author__ = "Ruslan Vaulin <vaulin@gravity.phys.uwm.edu>"
__version__ = "$Revision: 1.1 $"[11:-2]
__date__ = "$Date: 2012/01/21 04:15:29 $"[7:-2]

__prog__="gwem_swift_cwb_simulator.py"
__Id__ = "$Id: gwem_swift_cwb_simulator_10pxls.py,v 1.1 2012/01/21 04:15:29 uid40494 Exp $"


#loading standard modules
from optparse import *
import ConfigParser
import glob
import sys
import os 
import combine_multiple_searches_utils
from pylal import skylocutils
import numpy
numpy.seterr(all='ignore')

import matplotlib
matplotlib.use('Agg')
import pylab

from glue.ligolw import table
from glue.ligolw import lsctables
from glue.ligolw import ligolw
from glue.ligolw import utils
from pylal import InspiralUtils
import gwem_utils
import lr_utils
import model_fit
import scipy.interpolate 
import pickle
import scipy.optimize
import scipy.special
import numpy.random
import copy 

def adjacent(cluster1, cluster2):
  """
  Returns True if cluster1 and cluster2 may be merged.
  """
  
  adj = False
  
  for pixel1 in cluster1:
    for pixel2 in cluster2:
      if abs(pixel1[0] - pixel2[0]) <= 0.41:
        if min(abs(pixel1[1] - pixel2[1]), 360 - abs(pixel1[1] - pixel2[1])) * numpy.cos(pixel1[0]*numpy.pi/180.0) <= 0.41:
          adj = True
          break
          
  return adj



def cluster_pixels(pixels):
  """
  Clusters adjacent SWIFT pixels. A pixel is a tuple (dec, ra, prob)
  """

  clusters = [[pixel] for pixel in pixels]

  
  this_cluster_index = 0
  while this_cluster_index < len(clusters) - 1:
    
    n_clusters_before = len(clusters)
    n_clusters_after = 0
    while (n_clusters_before - n_clusters_after) > 0:
      n_clusters_before = len(clusters)
      next_cluster_index = this_cluster_index + 1
      while next_cluster_index < len(clusters):
          if adjacent(clusters[this_cluster_index], clusters[next_cluster_index]):
            clusters[this_cluster_index].extend(clusters[next_cluster_index])
            clusters.pop(next_cluster_index)
          else:
            next_cluster_index += 1
      n_clusters_after = len(clusters)
    this_cluster_index += 1
    
  max_probs = []
  sum_probs = []
    
  for cluster in clusters:
    max_prob = 0.0
    sum_prob = 0.0
    for pixel in cluster:
      sum_prob += pixel[2]
      if pixel[2] > max_prob:
        max_prob = pixel[2]
    max_probs.append(max_prob)
    sum_probs.append(sum_prob)
  
  return max_probs, sum_probs, clusters
  


def read_skymap_background(file):
  """ 
  Reads skymap probabilities from cwb background file.
  """
  bg_file = open(file, "r")
  
  bg_skymaps = []
  bg_gw_stats = []
  for (i, line) in enumerate(bg_file):
    values = line.split(" ")
    
    for  j in [15, 18, 21, 24, 27, 30, 33, 36, 39, 42]:
      if float(values[j]) > 0:
        bg_skymaps.append(float(values[j]))
        bg_gw_stats.append(float(values[5]))
    
        
  bg_file.close()     
  return bg_skymaps, bg_gw_stats,


def read_sum_skymap_background(file):
  """ 
  Reads skymap probabilities from cwb background file and sums over all pixels.
  """
  bg_file = open(file, "r")
  
  bg_gw_stats = []
  bg_sum_pms = []
  for (i, line) in enumerate(bg_file):
    values = line.split(" ")
    bg_sum_pms.append(float(values[6]))
    bg_gw_stats.append(float(values[5]))
        
  bg_file.close()      
  return bg_gw_stats, bg_sum_pms



def read_cluster_skymap_from_background(file):
  """
  Reads SWIFT skymaps from background file, identify clusters of pixels and return their probabiities.
  """
  bg_file = open(file, "r")
  
  bg_max_probs = []
  bg_sum_probs = []
  bg_gw_stats = []
  
  for (i, line) in enumerate(bg_file):
    values = line.split(" ")
     
    pixels = []
    for j in range(10):
      pixels.append((float(values[13+3*j]), float(values[14+3*j]), float(values[15+3*j])))
    
    event_max_probs, event_sum_probs, event_clusters = cluster_pixels(pixels)
  
    for j in range(len(event_max_probs)):
      bg_max_probs.append(event_max_probs[j])
      bg_sum_probs.append(event_sum_probs[j])
      bg_gw_stats.append(float(values[5]))
   
  bg_file.close()       
  return bg_gw_stats, bg_max_probs, bg_sum_probs
  
  
  
  
def read_cluster_skymap_from_injection(file, inj_sub_classes):
  """
  Reads SWIFT skymaps from injection file, identify clusters of pixels and return their probabiities.
  """
  inj_file = open(file, "r")
  
  inj_max_probs = []
  inj_sum_probs = []
  inj_gw_stats = []
  
  for (i, line) in enumerate(inj_file):
    values = line.split(" ")
    if int(values[9]) == 1 and ( (values[2], float(values[1])) in inj_sub_classes): 
      pixels = []
      for j in range(10):
        pixels.append((float(values[13+3*j]), float(values[14+3*j]), float(values[15+3*j])))
      
      event_max_probs, event_sum_probs, event_clusters = cluster_pixels(pixels)
     
      inj_pixel = (float(values[43]), float(values[44]), float(values[45])) 
      
      for j in range(len(event_max_probs)):
        #if inj_pixel in event_clusters[j]:
        inj_max_probs.append(event_max_probs[j])
        inj_sum_probs.append(event_sum_probs[j])
        inj_gw_stats.append(float(values[5]))
   
  inj_file.close()       
  return inj_gw_stats, inj_max_probs, inj_sum_probs
  
  
  
  


#######################################################################################################################################
###### Main Program
########################################################################################################################################

	  
usage= """

This code constructs likelihood ratio statistic for cwb+SWIFT injections and estimates ROC curve of the resulting search.
"""
###############################################################################
# Options
###############################################################################

def parse_command_line():

  """
  Parser function dedicated
  """
  
  # options for input data
  parser = OptionParser( usage=usage, version="version_1.0")
  
  parser.add_option("", "--burst-background",action="store",type="string",\
	  metavar="CWBFILE",help="file with cWb triggers")
      
  parser.add_option("", "--injections",action="store",type="string",\
	  metavar="INJDIR",help="injection directory")
  parser.add_option("", "--cwb-fap-params-file",action="store",type="string",\
	  metavar="",help="file with parameters of FAP distribution for  cwb likelihood ratio")      
  parser.add_option("", "--cwb-lr-params-file",action="store",type="string",\
	  metavar="",help="file with parameters needed to calculate cwb likelihood ratio")            
  parser.add_option("","--sky-map-investigation",action="store_true",\
      default=False,  metavar="",\
      help="include plots of alternative sky-map distributions that was investigated.")
      
  parser.add_option("", "--swift-catalog",action="store",type="string",\
	  metavar="SWIFTCATALOG",help="catalog of SWIFT serendipitous sources")     
      
  parser.add_option("", "--model-em-flux",action="store",type="float",\
	  metavar="modelemflux",help="Flux for a model of X-ray EM conterpart in units of 10^(-15) erg cm^2 s-1.")
      	       
  parser.add_option("", "--model-em-distance",action="store",type="float",\
	  metavar="modelemdistance",help="Reference distance for a model of X-ray EM conterpart in Mpc.")
                
  parser.add_option("-P","--output-path",action="store",\
      type="string",default="",  metavar="PATH",\
      help="path where the figures would be stored")
	  
  parser.add_option("-O","--enable-output",action="store_true",\
      default="True",  metavar="OUTPUT",\
      help="enable the generation of the html and cache documents")	 
	  
  parser.add_option("-u","--user-tag",action="store",type="string",\
      default="" ,metavar=" USERTAG",  \
      help="a user tag for the output filenames" )

  parser.add_option("", "--figure-resolution",action="store",type="int",\
      default=50, help="dpi of the thumbnails (50 by default)")
	  
	  
  parser.add_option("", "--html-for-cbcweb",action="store",\
      default=False, metavar = "CVS DIRECTORY", help="publish the html "\
      "output in a format that can be directly published on the cbc webpage "\
      "or in CVS. This only works IF --enable-output is also specified. The "\
      "argument should be the cvs directory where the html file will be placed "\
      "Example: --html-for-cbcweb protected/projects/s5/yourprojectdir")
  	  
  parser.add_option("","--verbose", action="store_true",\
      default=False, help="print information" )
	  
  (opts,args) = parser.parse_args()

  return opts, sys.argv[1:]
#####################################################################
opts, args = parse_command_line()
##############################################################################

if opts.verbose:
  print "Construct distributions for sky-maps probabilities:"

###########################################################################################
# Construct distributions for sky-map probabilities for background and injections
############################################################################################

# Based on the investigations we found that summing sky-map probabilities over 5 SWIFT pixels leads to 
# the most consistent distribution. Retaining the highest probability pixel leads to outliers for 
# even very high snr injections due to misidentification of its location by a pixel. Iditifying clusters
# of pixels does not lead to much of improvment. Injections do not give rise to bigger clusters than background.
# Below we keep the options we investigated but discarded. They can be trun on by giving option --sky-map-investigation.

# read background
if opts.verbose:
  print "  reading CWB background ..."
bg_stats, bg_sum_pms = read_sum_skymap_background(opts.burst_background)

if opts.verbose:
  print "  constructing histograms and estimating distribution for CWB background sky-maps ..."
bg_sum_pms = numpy.asarray(bg_sum_pms)

# sum over 5 pixel maps
bg_sum_pms_n, bg_sum_pms_bin_centres = lr_utils.weighted_histogram(bg_sum_pms,weight=None, norm=False, nbins=20)

bg_sum_pms_bin = (bg_sum_pms_bin_centres[1] - bg_sum_pms_bin_centres[0])
bg_sum_pms_hist_norm = numpy.sum(bg_sum_pms_n)*(bg_sum_pms_bin_centres[1] - bg_sum_pms_bin_centres[0])

#smooth gaussian kernel pdf
bg_sum_pms_step = (bg_sum_pms_bin_centres[1] - bg_sum_pms_bin_centres[0])/10.0
bg_sum_pms_x = numpy.arange(bg_sum_pms.min(), bg_sum_pms.max()+ bg_sum_pms_step, bg_sum_pms_step)
bg_sum_pms_w = 1.0*(bg_sum_pms_bin_centres[1] - bg_sum_pms_bin_centres[0])
bg_sum_pms_pdf = [skylocutils.gaussian_kde(bg_sum_pms,x,bg_sum_pms_w) for x in bg_sum_pms_x]

bg_sum_pms_pdf= numpy.asarray(bg_sum_pms_pdf)

bg_sum_pms_pdf_norm = numpy.sum(bg_sum_pms_pdf)*bg_sum_pms_step


# define interpolationt curve for background  which will be used to calculate FAP
bg_sum_pms_curve = scipy.interpolate.interp1d(bg_sum_pms_x, bg_sum_pms_pdf/bg_sum_pms_pdf_norm, kind="linear")


if opts.sky_map_investigation:

  bg_skymaps, bg_gw_stats = read_skymap_background(opts.burst_background)


  bg_cl_stats, bg_cl_max_probs, bg_cl_sum_probs = read_cluster_skymap_from_background(opts.burst_background)



  # histogram

  # one pixel maps
  bg_pm_n, bg_pm_bin_centres = lr_utils.weighted_histogram(bg_skymaps,weight=None, norm=False, nbins=20)

  bg_pm_bin = (bg_pm_bin_centres[1] - bg_pm_bin_centres[0])
  bg_pm_hist_norm = numpy.sum(bg_pm_n)*(bg_pm_bin_centres[1] - bg_pm_bin_centres[0])


  # sum and max over clusters maps

  bg_cl_max_n, bg_cl_max_bin_centres = lr_utils.weighted_histogram(bg_cl_max_probs,weight=None, norm=False, nbins=20)
  bg_cl_max_bin = (bg_cl_max_bin_centres[1] - bg_cl_max_bin_centres[0])
  bg_cl_max_hist_norm = numpy.sum(bg_cl_max_n)*(bg_cl_max_bin_centres[1] - bg_cl_max_bin_centres[0])

  bg_cl_sum_n, bg_cl_sum_bin_centres = lr_utils.weighted_histogram(bg_cl_sum_probs,weight=None, norm=False, nbins=20)
  bg_cl_sum_bin = (bg_cl_sum_bin_centres[1] - bg_cl_sum_bin_centres[0])
  bg_cl_sum_hist_norm = numpy.sum(bg_cl_sum_n)*(bg_cl_sum_bin_centres[1] - bg_cl_sum_bin_centres[0])


if opts.verbose:
  print "  reading in injections ..."


# read injections
all_injections = gwem_utils.read_gwem_injections(opts.injections)


if opts.verbose:
  print "  constructing histograms and estimating  distribution for CWB injection sky-maps ..." 
# get those that were succesfully localized 
injections = all_injections.get_localized()

if opts.verbose:
  print  str(len(injections)) + " out of " + str(len(all_injections)) + " have been localized by Swift observations." 

inj_sum_pms = injections.get_column("prob_in_top_5_swift_pxls")

inj_sum_pms_n, inj_sum_pms_bin_centres = lr_utils.weighted_histogram(inj_sum_pms,weight=None, norm=False, nbins=20)

inj_sum_pms_bin = (inj_sum_pms_bin_centres[1] - inj_sum_pms_bin_centres[0])
inj_sum_pms_hist_norm = numpy.sum(inj_sum_pms_n)*(inj_sum_pms_bin_centres[1] - inj_sum_pms_bin_centres[0])

#smooth gaussian kernel pdf
inj_sum_pms_step = (inj_sum_pms_bin_centres[1] - inj_sum_pms_bin_centres[0])/10.0
inj_sum_pms_x = numpy.arange(inj_sum_pms.min(), inj_sum_pms.max()+ inj_sum_pms_step, inj_sum_pms_step)
inj_sum_pms_w = 1.0*(inj_sum_pms_bin_centres[1] - inj_sum_pms_bin_centres[0])
inj_sum_pms_pdf = [skylocutils.gaussian_kde(inj_sum_pms,x,inj_sum_pms_w) for x in inj_sum_pms_x]

inj_sum_pms_pdf= numpy.asarray(inj_sum_pms_pdf)

inj_sum_pms_pdf_norm = numpy.sum(inj_sum_pms_pdf)*inj_sum_pms_step


# define interpolationt curve for background  which will be used to calculate FAP
inj_sum_pms_curve = scipy.interpolate.interp1d(inj_sum_pms_x, inj_sum_pms_pdf/inj_sum_pms_pdf_norm, kind="linear")




if opts.sky_map_investigation:

  inj_skymaps = injections.get_column("sky_map_prob")
  # histogram
  inj_pm_n, inj_pm_bin_centres = lr_utils.weighted_histogram(inj_skymaps,weight=None, norm=False, nbins=20)

  inj_pm_bin = (inj_pm_bin_centres[1] - inj_pm_bin_centres[0])
  inj_pm_hist_norm = numpy.sum(inj_pm_n)*(inj_pm_bin_centres[1] - inj_pm_bin_centres[0])


  # injection cluster maps

  inj_sub_classes = []
  for inj in injections:
    inj_sub_classes.append((inj.type, inj.hrss_fac))


  inj_cl_stats, inj_cl_max_probs, inj_cl_sum_probs = read_cluster_skymap_from_injection("/home/vaulin/analysis/GW_EM/SWIFT/data/cwb.inj10.rev.txt", inj_sub_classes)

  # sum and max over clusters maps

  inj_cl_max_n, inj_cl_max_bin_centres = lr_utils.weighted_histogram(inj_cl_max_probs,weight=None, norm=False, nbins=20)
  inj_cl_max_bin = (inj_cl_max_bin_centres[1] - inj_cl_max_bin_centres[0])
  inj_cl_max_hist_norm = numpy.sum(inj_cl_max_n)*(inj_cl_max_bin_centres[1] - inj_cl_max_bin_centres[0])

  inj_cl_sum_n, inj_cl_sum_bin_centres = lr_utils.weighted_histogram(inj_cl_sum_probs,weight=None, norm=False, nbins=20)
  inj_cl_sum_bin = (inj_cl_sum_bin_centres[1] - inj_cl_sum_bin_centres[0])
  inj_cl_sum_hist_norm = numpy.sum(inj_cl_sum_n)*(inj_cl_sum_bin_centres[1] - inj_cl_sum_bin_centres[0])


if opts.verbose:
  print "Done"

################################################################################################################
# Background for SWIFT
###############################################################################################################

if opts.verbose:
  print "Estimate Background for SWIFT:"

if opts.verbose:
  print "  reading in SWIFT catalog ..."
# read in the catalog
# flux units erg cm^2 s-1
flux_scale = 10.0**(16)


# read bg fluxes from file
if opts.swift_catalog.endswith("pickle"):
  #swift_bg_fluxes = pickle.load(open(opts.swift_catalog, "r"))
  (swift_hgl_sources, swift_lgl_sources) = pickle.load(open(opts.swift_catalog, "r"))
  
else:

  #swift_bg_fluxes = gwem_utils.read_swift_catalog(opts.swift_catalog)*flux_scale

  swift_bg_fluxes = gwem_utils.read_xmmi_catalog(opts.swift_catalog, "/home/vaulin/analysis/GW_EM/SWIFT/data/load_2XMMi-DR3cat_v1.0.sql.txt")*flux_scale

  # save bg fluxes into a file

  bg_fluxes_file = open("bg_fluxes.pickle", "w")
  pickle.dump(swift_bg_fluxes, bg_fluxes_file)
  bg_fluxes_file.close()


# get fluxes
swift_hgl_fluxes = []
for source in swift_hgl_sources:
  swift_hgl_fluxes.append(source[0])
swift_hgl_fluxes = numpy.asarray(swift_hgl_fluxes)

swift_lgl_fluxes = []
for source in swift_lgl_sources:
  swift_lgl_fluxes.append(source[0])
swift_lgl_fluxes = numpy.asarray(swift_lgl_fluxes)




swift_hgl_fluxes *= flux_scale
swift_lgl_fluxes *= flux_scale

swift_bg_fluxes = numpy.concatenate((swift_hgl_fluxes, swift_lgl_fluxes))


if opts.verbose:
  print "  constructing histograms and estimating background distribution for SWIFT ..."
# histogram
swift_bg_n, swift_bg_bin_centres = lr_utils.weighted_histogram(numpy.log10(swift_bg_fluxes),weight=None, norm=False, nbins=50)

swift_bg_bin = (swift_bg_bin_centres[1] - swift_bg_bin_centres[0])
swift_bg_hist_norm = numpy.sum(swift_bg_n)*(swift_bg_bin_centres[1] - swift_bg_bin_centres[0])


#smooth gaussian kernel pdf
swift_bg_step = abs(numpy.log10(numpy.max(swift_bg_fluxes)) - numpy.log10(numpy.min(swift_bg_fluxes)))/1000.0
swift_bg_x = numpy.arange(numpy.log10(numpy.min(swift_bg_fluxes)), numpy.log10(numpy.max(swift_bg_fluxes))+ swift_bg_step, swift_bg_step)
swift_bg_w = 30.0*swift_bg_step
swift_bg_pdf = [skylocutils.gaussian_kde(numpy.log10(swift_bg_fluxes),x,swift_bg_w) for x in swift_bg_x]

swift_bg_pdf= numpy.asarray(swift_bg_pdf)

swift_bg_pdf_norm = numpy.sum(swift_bg_pdf)*swift_bg_step

# Find parameters for analytical model for logN-Log(flux) curve: N(S'>S) = K/S^alpha1 + S0^alpha1, where S = log(flux).
swift_bg_par = model_fit.swift_onepower_fit(swift_bg_bin_centres, swift_bg_n)

swift_bg_power_n = model_fit.swift_onepower(swift_bg_par, swift_bg_x)
swift_bg_power_norm = numpy.sum(swift_bg_power_n)*swift_bg_step
 
right_cum_prob = (swift_bg_par[2]/swift_bg_power_norm)/(10**(swift_bg_par[0]*swift_bg_x[-1]) + swift_bg_par[1]**swift_bg_par[0])

#############################################################################################
#  Calculate False Alarm Probability for joined CWB-SWIFT likelihood ratio
#############################################################################################

#temporarily fix scale for swift likelihood ratio to be 1 at the maximum of background pdf.
swift_stat_scale = numpy.exp(1)*numpy.max(swift_bg_power_n/swift_bg_power_norm)
# add likelhood ratio scale as a fourth swift parameter and normalize swift curve P(S_min) = 1.
swift_params = [swift_bg_par[0], swift_bg_par[1], swift_bg_par[2]/swift_bg_power_norm, swift_stat_scale]

# normalization of swift background distribution
# we determine it by finding the flux at which the rate of events is one per observation.
# observation covers 5 pixels 0.4x0.4 deg^2

# for swift catatlog
#catalog_sky_coverage = 22.12

#for 2xmmi catalog
catalog_sky_coverage = 504.0

swift_observation = 10.0*0.4*0.4
R_total = swift_observation*len(swift_bg_fluxes)/catalog_sky_coverage 

# get flux at which the rate ( average number of sources with flux higher or equal) is one source per observation.
# the function below is searching for x at which SWIFT_cum_dist_opt_func(x) = 0.
# this corresponds to sloving P(S) - 1.0/R_total = 0 for S.
# To see that this is the case, recall that P(S)=N(s)/N_total
#then P(S)*R_total = N(s)*S_obs*N_total/(N_total*S_total) =N(s)*S_obs/S_total = n_obs(S) which is the average number of sources per observation. 
# n_obs(S) = 1.0 leads to this convoluted equation P(S) - 1.0/R_total = 0.
unit_flux = scipy.optimize.fsolve(gwem_utils.SWIFT_cum_dist_opt_func, 2.0, args = (swift_params[:-1], 1.0/R_total))

# renormalizing swift curve so that P(S_unit) = 1 - exp(-1.0) accoridng to Poisson distribution. 
renormalized_K = (1.0 - numpy.exp(-1))/ gwem_utils.SWIFT_cum_dist(unit_flux, [swift_params[0], swift_params[1], 1.0])

# set the normalization factor K to the new one.
swift_params[2] = renormalized_K

# find the minimum flux for which probability of observing an event is one.
# This is the threshold flux for the search. Any source with flux below it is dismissed.   
min_log_flux = scipy.optimize.fsolve(gwem_utils.SWIFT_cum_dist_opt_func, 2.0, args = (swift_params[:-1], 1.0))  

if opts.verbose:
  print "Minimum cut-off flux for accidental X-ray sources: ", 10.0**(min_log_flux) / flux_scale 

# adjust the scale so that log likelihood for swift is one for the lowest flux.
swift_stat_scale = numpy.exp(1)*gwem_utils.swift_flux_stat_func(min_log_flux, [swift_params[0],swift_params[1], swift_params[2], 0.0], 1.0, 0.0)
swift_params[3] = swift_stat_scale

if opts.verbose:
  print "  Distribution of swift is modelled as P(S) = K/(S^alpha1 + S0^alpha1), where S is log_10(flux) and P(S) is cumulative probability."
  print "  SWIFT Likelihood ratio is defined as C/p(s), where C is the scale that is set by requiring that ln(C/p(s_min)) = 1, P(s_min) = 1."
  print "  Parameters (alpha1, S0 , K, stat_scale) for analytic fit for SWIFT background are:", swift_params


if opts.verbose:
  print "  checking  how well swift fap function works ..." 

# find index in array of fluxes corresponding to min flux.
min_index = numpy.searchsorted(swift_bg_x, min_log_flux)
#renormalize analytical distribution so that it would correcpond to P(S_min) = 1.0. 
#swift_bg_power_norm_obs = numpy.sum(swift_bg_power_n[min_index:])*swift_bg_step/ (1.0 - gwem_utils.SWIFT_cum_dist(swift_bg_x[-1], swift_params[:-1]))

# check how well SWIFT_FAP works
# Define rho_e = ln(C/p(s)), then P(rho_e) = P(s) which is the swift background distribution.
# This is so because p(s) is monotonicly decreasing with s. To calculate P(rho_e), one need to find s=ln(flux) corresponding to rho_e.
# so one need to solve rho_e - ln(C/p(s)) = 0 for s, and plug it in P(s) - the cumulative probability formula. 
# Below we check how well this process work.

# form erray of rho_e using model pdf function for swift background and equally sampled values of log_10(flux), starting from the minimum flux.
rho_e_array = numpy.log(swift_stat_scale / model_fit.swift_onepower(swift_params[:-1], swift_bg_x[min_index:]))

swift_faps = []
swift_x = []
#calculate FAP for each value of rho_e
for rho_e in rho_e_array:
  (fap, x) = gwem_utils.SWIFT_FAP(rho_e, swift_params, min_log_flux-0.5)
  swift_faps.append(fap)
  swift_x.append(x)
  
swift_faps = numpy.asarray(swift_faps)

# compare the result of numerical inversion in function SWIFT_FAP with analytical formula given by SWIFT_cum_dist for the same array of fluxes.
fap_rel_error = (swift_faps - gwem_utils.SWIFT_cum_dist(swift_bg_x[min_index:], swift_params[:-1]))/ gwem_utils.SWIFT_cum_dist(swift_bg_x[min_index:], swift_params[:-1])


if opts.verbose:
  print "  relative errors do not exceed: ", numpy.max(numpy.abs(fap_rel_error))
# end of check

if opts.verbose:
  print "Done."

if opts.verbose:
  print "Estimate joint CWB+SWIFT FAP background:"
  
if opts.verbose:
  print "  reading in parameters for cwb background distribution for likeloihhod ratio ..." 
   
#read in parameters for background distribution of rho_g = ln(cwb likelihood)

fap_file = open(opts.cwb_fap_params_file, "r")
cwb_bg_lr_par = pickle.load(fap_file)
fap_file.close()

cwb_params = [cwb_bg_lr_par[1], cwb_bg_lr_par[2]/numpy.exp(cwb_bg_lr_par[0])]


#The joint statistic is defined as rho_joint = rho_g + rho_e
# Set the minimum values for rho_g and rho_e
rho_e_min = 1.0
rho_g_min = 1.0

# as a check calculate FAP for rho_joint = 4.0
FAP=gwem_utils.CWB_SWIFT_FAP(4.0, swift_params, cwb_params, rho_e_min, rho_g_min, min_log_flux-0.5)

if opts.verbose:
  print "  FAP for joint log likelihhod ratio of 4.0 is ", FAP


if opts.verbose:
  print "Done."

#################### calculate and assign the joint likelihood for injections  ############################################

if opts.verbose:
  print "Calculate and assign joint likelihood ratio and FAP to injections:" 


#read in flux and distance of the model em swift counterpart
model_em_flux = numpy.log10(opts.model_em_flux)
model_em_distance = opts.model_em_distance

if opts.verbose:
  " assigning X-ray flux to each injection using the model ..."
# use the model em swift counterpart to set flux magnitude for em swift counterpart for each injection
# model flux gets scaled by distance to the source
all_injections.populate_em_flux_column(model_em_flux, model_em_distance)

# read in parameters needed for calculating cwb likelihood ratio from file

lr_file = open(opts.cwb_lr_params_file, "r")
(inj_x, cwb_inj_pdf) = pickle.load(lr_file)
bg_exppowpar = pickle.load(lr_file)
LR_scale = pickle.load(lr_file)
lr_file.close()

# define interpolationt curve for injections which will be used to calculate cwb likelihood ratio
inj_curve = scipy.interpolate.interp1d(inj_x, cwb_inj_pdf, kind="linear")

if opts.verbose:
  print "  calculating gw likelihoo ratio for each injection ..."
# calculate cwb likelihood ratio for each injection
all_injections.calculate_gw_lr(bg_exppowpar, inj_curve, LR_scale)

# get distance, stat etc from injections before applying any cuts. 
# eta for orginal injections goes as low as 3.0
original_injs_distance = all_injections.get_column("distance")
original_injs_stat = all_injections.get_column("gw_stat")
 

# keep only injections with cwb likelihood ratio above 1.0
# this corresponds to rho > 3.6
all_injections = all_injections.get_above_gw_lr_threshold(1.0) 

if opts.verbose:
  print "  calculating em likelihood ratio for each injection ..."
# calculate swift likelihood ratio for each injection
all_injections.calculate_em_lr(swift_params)

if opts.verbose:
  print "  calculating joint likelihood ratio for each injection ..."
# calculate joint GW+EM likelihood ratio for each injection
all_injections.calculate_joint_lr()

if opts.verbose:
  print "  calculating cwb FAP for each injection ..."
# calculate CWB FAP
all_injections.calculate_gw_fap(cwb_params)

if opts.verbose:
  print "  calculating swift FAP for each injection ..."
#calculate SWIFT FAP
all_injections.calculate_em_fap(swift_params, min_log_flux-0.5)

if opts.verbose:
  print "  calculating the joint FAP for each injection ..."
#calculate joint FAP 
all_injections.calculate_joint_fap(swift_params, cwb_params, rho_e_min, rho_g_min, min_log_flux - 0.5)

if opts.verbose:
  print "Done."


# check for joint FAP
# construct joint background events from cwb and swift background events

# from swift events get those with flux above min flux

swift_bg_fluxes_above_min_flux = []
for flux in numpy.log10(swift_bg_fluxes):
  if flux >= min_log_flux:
    swift_bg_fluxes_above_min_flux.append(flux)

cwb_bg_lr_faps = []
for stat in bg_stats:
  gw_lr = numpy.log(LR_scale * gwem_utils.calculate_cwb_lr(stat, bg_exppowpar, inj_curve))
  fap = gwem_utils.calculate_cwb_fap(gw_lr, cwb_params)
  cwb_bg_lr_faps.append((gw_lr, fap))
  
em_bg_lr_faps = []
for flux in swift_bg_fluxes_above_min_flux:
  em_lr = numpy.log(gwem_utils.calculate_swift_lr(flux, swift_params))
  fap = gwem_utils.SWIFT_FAP(em_lr, swift_params, min_log_flux-0.5)
  em_bg_lr_faps.append((em_lr, fap))
  
# generate several background sample populations    
N_bg_trials = 10
#pair up randomly cwb with swift sources

joint_bg_trials = {}
for trial in range(N_bg_trials):

  joint_bg_events = []
  #for stat in bg_stats:
  #  joint_bg_events.append((stat, swift_bg_fluxes_above_min_flux[numpy.random.random_integers(0,len(swift_bg_fluxes_above_min_flux)-1)]))
  
  for i in range(min(len(swift_bg_fluxes_above_min_flux), len(bg_stats))):
    flux = swift_bg_fluxes_above_min_flux[numpy.random.random_integers(0,len(swift_bg_fluxes_above_min_flux)-1)]
    bg_stat = bg_stats[numpy.random.random_integers(0,len(bg_stats)-1)]
    joint_bg_events.append((bg_stat, flux))
  
  joint_bg_trials[trial] = joint_bg_events
  # calculate joint likelihood for background events

  bg_joint_lr = []
  lr_for_joint_events = []
  for event in joint_bg_events:
    gw_lr = numpy.log(LR_scale * gwem_utils.calculate_cwb_lr(event[0], bg_exppowpar, inj_curve))
    em_lr = numpy.log(gwem_utils.calculate_swift_lr(event[1], swift_params))
    lr_for_joint_events.append((gw_lr, em_lr)) 
    bg_joint_lr.append(gw_lr + em_lr)

  # calculate joint FAP for these events

  bg_joint_lr_faps = []
  for joint_lr in bg_joint_lr:
    bg_joint_lr_faps.append((joint_lr, gwem_utils.CWB_SWIFT_FAP(joint_lr, swift_params, cwb_params, rho_e_min, rho_g_min, min_log_flux-0.5)))
    
  #bg_em_lr_faps = []   
  #for (gw_lr,em_lr) in lr_for_joint_events:
  #  bg_em_lr_faps.append((em_lr, gwem_utils.SWIFT_FAP(em_lr, swift_params, min_log_flux-0.5)))  

  #bg_gw_lr_faps = []   
  #for (gw_lr,gw_lr) in lr_for_joint_events:
  #  bg_gw_lr_faps.append((gw_lr, gwem_utils.calculate_cwb_fap(gw_lr, cwb_params)))  
    

  joint_bg_trials[trial] = (joint_bg_events, lr_for_joint_events, bg_joint_lr_faps)
#################### Calculate ROC curves for injections  #########################################

if opts.verbose:
  print "Computing ROC curves"
# compute ROC curve using gw fap
(all_gw_fap, all_gw_eff) = gwem_utils.ROC(all_injections, "gw_fap", localized_only = False)


#compute ROC curve for joint search, using joint fap

# only for successfully localized injections
(localized_joint_fap, localized_joint_eff) = gwem_utils.ROC(all_injections, "joint_fap", localized_only = True)

# for all injections
(all_joint_fap, all_joint_eff) = gwem_utils.ROC(all_injections, "joint_fap", localized_only = False)
  
if opts.verbose:
  print "Done."


# for injections that were not localized assign em_lr=1 and use in ROC curve

tmp_injections = copy.deepcopy(all_injections)

for injection in tmp_injections:
  if injection.in_top_5_swift_pxls != 1:
    injection.em_lr = 1.0
    injection.joint_lr = injection.gw_lr + injection.em_lr
    injection.em_fap = 1.0
    injection.joint_fap = gwem_utils.CWB_SWIFT_FAP(injection.joint_lr, swift_params, cwb_params, rho_e_min, rho_g_min, min_log_flux)


(hybrid_joint_fap, hybrid_joint_eff) = gwem_utils.ROC(tmp_injections, "joint_fap", localized_only = False)


# save ROC curves into file
roc_file = open("roc_curves_" + opts.user_tag + ".pickle", "w")
pickle.dump((all_gw_fap, all_gw_eff), roc_file)
pickle.dump((localized_joint_fap, localized_joint_eff), roc_file)
pickle.dump((all_joint_fap, all_joint_eff), roc_file)
pickle.dump((hybrid_joint_fap, hybrid_joint_eff), roc_file)
roc_file.close()


################   PLOTS   #############################################################


# plotting

# Initializing the html output
InspiralUtils.message(opts, "Initialisation...")
opts = InspiralUtils.initialise(opts, __prog__, __version__)
fnameList = []
tagList = []
fig_num = 0
comments = ""


######### plots of skymap probabilities    ##############

# background and inections skymap probs vs rho

fig_num += 1
pylab.figure(fig_num)
legend_text = []
pylab.semilogx(bg_stats, bg_sum_pms, 'k+', label="background")
pylab.semilogx(injections.get_column("gw_stat"), inj_sum_pms, 'rx', label="injections")
pylab.xlabel(r"$\eta$")
pylab.title("Fig. "+str(fig_num)+": Sky-map summed over observed pixels vs cwb statistic.")
pylab.legend()

name = "sum_skymap_vs_rho" 
fname = InspiralUtils.set_figure_name(opts, name)
fname_thumb = InspiralUtils.savefig_pylal(filename=fname, doThumb=True, dpi_thumb=opts.figure_resolution)
fnameList.append(fname)
tagList.append("Sum of Skymap probability vs cwb statistic")
pylab.close()


# histograms of skymap probs for background and injections

fig_num += 1
pylab.figure(fig_num)
legend_text = []
pylab.plot(bg_sum_pms_bin_centres, bg_sum_pms_n/bg_sum_pms_hist_norm, "k^", label="background histogram")
pylab.plot(bg_sum_pms_x, bg_sum_pms_pdf/bg_sum_pms_pdf_norm, color="k", linestyle="-", label="background smooth pdf")
pylab.plot(inj_sum_pms_bin_centres, inj_sum_pms_n/inj_sum_pms_hist_norm, "ro", label="injections histogram")
pylab.plot(bg_sum_pms_x, inj_sum_pms_pdf/inj_sum_pms_pdf_norm, color="r", linestyle="-", label="injections smooth pdf")
pylab.xlabel( "sky-map probability summed over SWIFT pixels")
pylab.title("Fig. "+str(fig_num)+": Normalized PDF for sky-map summed over observed pixels")
pylab.legend()

name = "sum_sky_map_pdfs" 
fname = InspiralUtils.set_figure_name(opts, name)
fname_thumb = InspiralUtils.savefig_pylal(filename=fname, doThumb=True, dpi_thumb=opts.figure_resolution)
fnameList.append(fname)
tagList.append("Normalized distributions of sum sky-map probabilities")
pylab.close()

# histograms of skymap probs for background and injections


if opts.sky_map_investigation:

  # background skymap probs vs rho

  fig_num += 1
  pylab.figure(fig_num)
  legend_text = []
  pylab.plot(bg_gw_stats, bg_skymaps, 'k+', label="background")
  pylab.xlabel(r"$\eta$")
  pylab.title("Fig. "+str(fig_num)+": Sky-map pixels for background vs cwb statistic")
  pylab.legend()

  name = "bg_skymap_vs_rho" 
  fname = InspiralUtils.set_figure_name(opts, name)
  fname_thumb = InspiralUtils.savefig_pylal(filename=fname, doThumb=True, dpi_thumb=opts.figure_resolution)
  fnameList.append(fname)
  tagList.append("Background skymap probability vs cwb statistic")
  pylab.close()




  # background and inections skymap probs vs rho

  fig_num += 1
  pylab.figure(fig_num)
  legend_text = []
  pylab.semilogx(bg_gw_stats, bg_skymaps, 'k+', label="background")
  pylab.semilogx(injections.get_column("gw_stat"), inj_skymaps, 'rx', label="injections")
  pylab.xlabel(r"$\eta$")
  pylab.title("Fig. "+str(fig_num)+": Sky-map pixels vs cwb statistic")
  pylab.legend()

  name = "skymap_vs_rho" 
  fname = InspiralUtils.set_figure_name(opts, name)
  fname_thumb = InspiralUtils.savefig_pylal(filename=fname, doThumb=True, dpi_thumb=opts.figure_resolution)
  fnameList.append(fname)
  tagList.append("Skymap probability vs cwb statistic")
  pylab.close()


  # histograms of skymap probs for background and injections

  fig_num += 1
  pylab.figure(fig_num)
  legend_text = []
  pylab.plot(bg_pm_bin_centres, bg_pm_n/bg_pm_hist_norm, "k^", label="background histogram")
  pylab.plot(inj_pm_bin_centres, inj_pm_n/inj_pm_hist_norm, "ro", label="injections histogram")
  pylab.xlabel("sky-map pixel")
  pylab.title("Fig. "+str(fig_num)+": Normalized PDF of sky-map pixels.")
  pylab.legend()

  name = "sky_map_pdfs" 
  fname = InspiralUtils.set_figure_name(opts, name)
  fname_thumb = InspiralUtils.savefig_pylal(filename=fname, doThumb=True, dpi_thumb=opts.figure_resolution)
  fnameList.append(fname)
  tagList.append("Normalized distributions of sky-map probabilities")
  pylab.close()




  # cluster max plots

  # background sum skymap probs vs rho

  fig_num += 1
  pylab.figure(fig_num)
  legend_text = []
  pylab.plot(bg_cl_stats, bg_cl_max_probs, 'k+', label="background")
  pylab.xlabel(r"$\eta$")
  pylab.title("Fig. "+str(fig_num)+": Max sky-map pixel in cluster for background vs cwb statistic")
  pylab.legend()

  name = "bg_cl_max_skymap_vs_rho" 
  fname = InspiralUtils.set_figure_name(opts, name)
  fname_thumb = InspiralUtils.savefig_pylal(filename=fname, doThumb=True, dpi_thumb=opts.figure_resolution)
  fnameList.append(fname)
  tagList.append("Background max of probabilities in cluster vs cwb statistic")
  pylab.close()




  # background and inections skymap probs vs rho

  fig_num += 1
  pylab.figure(fig_num)
  legend_text = []
  pylab.semilogx(bg_cl_stats, bg_cl_max_probs, 'k+', label="background")
  pylab.semilogx(inj_cl_stats, inj_cl_max_probs, 'rx', label="injections")
  pylab.xlabel(r"$\eta$")
  pylab.title("Fig. "+str(fig_num)+": Max sky-map pixel in cluster for background vs cwb statistic")
  pylab.legend()

  name = "cl_max_skymap_vs_rho" 
  fname = InspiralUtils.set_figure_name(opts, name)
  fname_thumb = InspiralUtils.savefig_pylal(filename=fname, doThumb=True, dpi_thumb=opts.figure_resolution)
  fnameList.append(fname)
  tagList.append("Max of probabilities in cluster vs cwb statistic")
  pylab.close()


  # histograms of skymap probs for background and injections

  fig_num += 1
  pylab.figure(fig_num)
  legend_text = []
  pylab.plot(bg_cl_max_bin_centres, bg_cl_max_n/bg_cl_max_hist_norm, "k^", label="background histogram")
  pylab.plot(inj_cl_max_bin_centres, inj_cl_max_n/inj_cl_max_hist_norm, "ro", label="injections histogram")
  pylab.xlabel("max sky-map pixel in cluster")
  pylab.title("Fig. "+str(fig_num)+": Normalized PDF of max sky-map in cluster.")
  pylab.legend()

  name = "cl_max_sky_map_pdfs" 
  fname = InspiralUtils.set_figure_name(opts, name)
  fname_thumb = InspiralUtils.savefig_pylal(filename=fname, doThumb=True, dpi_thumb=opts.figure_resolution)
  fnameList.append(fname)
  tagList.append("Normalized distributions of max cluster sky-map probability")
  pylab.close()


  # cluster sum plots

  # background sum skymap probs vs rho

  fig_num += 1
  pylab.figure(fig_num)
  legend_text = []
  pylab.plot(bg_cl_stats, bg_cl_sum_probs, 'k+', label="background")
  pylab.xlabel(r"$\eta$")
  pylab.title("Fig. "+str(fig_num)+": Sky-map pixels summed over cluster vs cwb statistic")
  pylab.legend()

  name = "bg_cl_sum_skymap_vs_rho" 
  fname = InspiralUtils.set_figure_name(opts, name)
  fname_thumb = InspiralUtils.savefig_pylal(filename=fname, doThumb=True, dpi_thumb=opts.figure_resolution)
  fnameList.append(fname)
  tagList.append("Background sum of probabilities in cluster vs cwb statistic")
  pylab.close()




  # background and inections skymap probs vs rho

  fig_num += 1
  pylab.figure(fig_num)
  legend_text = []
  pylab.semilogx(bg_cl_stats, bg_cl_sum_probs, 'k+', label="background")
  pylab.semilogx(inj_cl_stats, inj_cl_sum_probs, 'rx', label="injections")
  pylab.xlabel(r"$\eta$")
  pylab.title("Fig. "+str(fig_num)+": Sky-map pixels summed over cluster vs cwb statistic")
  pylab.legend()

  name = "cl_sum_skymap_vs_rho" 
  fname = InspiralUtils.set_figure_name(opts, name)
  fname_thumb = InspiralUtils.savefig_pylal(filename=fname, doThumb=True, dpi_thumb=opts.figure_resolution)
  fnameList.append(fname)
  tagList.append("Sum of probabilities in cluster vs cwb statistic")
  pylab.close()


  # histograms of skymap probs for background and injections

  fig_num += 1
  pylab.figure(fig_num)
  legend_text = []
  pylab.plot(bg_cl_sum_bin_centres, bg_cl_sum_n/bg_cl_sum_hist_norm, "k^", label="background histogram")
  pylab.plot(inj_cl_sum_bin_centres, inj_cl_sum_n/inj_cl_sum_hist_norm, "ro", label="injections histogram")
  pylab.xlabel("sky-map pixels summed over cluster")
  pylab.title("Fig. "+str(fig_num)+": Normalized PDF of sky-map pixels summed over cluster.")
  pylab.legend()

  name = "cl_sum_sky_map_pdfs" 
  fname = InspiralUtils.set_figure_name(opts, name)
  fname_thumb = InspiralUtils.savefig_pylal(filename=fname, doThumb=True, dpi_thumb=opts.figure_resolution)
  fnameList.append(fname)
  tagList.append("Normalized distributions of sum cluster sky-map probability")
  pylab.close()



############# plots for swift background ######################################

# plot of number denisty for background

fig_num += 1
pylab.figure(fig_num)
legend_text = []
pylab.plot(swift_bg_x, swift_bg_power_n, color='b', linestyle="-", label="power fit")
pylab.plot(swift_bg_bin_centres, swift_bg_n, "ko", label="histogram")
pylab.axvline(x=min_log_flux, color='r', linestyle='--', label="cut-off flux")

pylab.xlabel(r"$\log_{10}(flux \times 10^{-16}),$ erg cm$^{-2}$ s$^{-1}$")
pylab.title("Fig. "+str(fig_num)+": Distribution of flux for serendipitous X-ray sources.")
pylab.legend()

name = "swift_background_density" 
fname = InspiralUtils.set_figure_name(opts, name)
fname_thumb = InspiralUtils.savefig_pylal(filename=fname, doThumb=True, dpi_thumb=opts.figure_resolution)
fnameList.append(fname)
tagList.append("SWIFT Background")
pylab.close()

# plot of cumulative number densities

fig_num += 1
pylab.figure(fig_num)
legend_text = []
pylab.plot(swift_bg_x, numpy.cumsum(swift_bg_power_n[::-1])[::-1]*swift_bg_step/swift_bg_bin, color='b', linestyle="-", label="power fit")
pylab.plot(swift_bg_bin_centres, numpy.cumsum(swift_bg_n[::-1])[::-1], "ko", label="histogram")
pylab.axvline(x=min_log_flux, color='r', linestyle='--', label="cut-off flux")

pylab.xlabel(r"$\log_{10}(flux \times 10^{-16}),$ erg cm$^{-2}$ s$^{-1}$")
pylab.title("Fig. "+str(fig_num)+": Cumulative distribution of flux for serendipitous X-ray sources.")
pylab.legend()

name = "swift_cum_background_density" 
fname = InspiralUtils.set_figure_name(opts, name)
fname_thumb = InspiralUtils.savefig_pylal(filename=fname, doThumb=True, dpi_thumb=opts.figure_resolution)
fnameList.append(fname)
tagList.append("Cumulative SWIFT background number density")
pylab.close()

#normalized distributions

fig_num += 1
pylab.figure(fig_num)
legend_text = []
pylab.plot(swift_bg_x, swift_bg_pdf/swift_bg_pdf_norm, color='k', label="smooth pdf")
pylab.plot(swift_bg_x, swift_bg_power_n/swift_bg_power_norm, color='b', linestyle="-", label="power fit")
pylab.plot(swift_bg_bin_centres, swift_bg_n/swift_bg_hist_norm, "ko", label="histogram")
pylab.axvline(x=min_log_flux, color='r', linestyle='--', label="cut-off flux")

pylab.xlabel(r"$\log_{10}(flux \times 10^{-16}),$ erg cm$^{-2}$ s$^{-1}$")
pylab.title("Fig. "+str(fig_num)+": Normalized flux PDF  for serendipitous X-ray sources.")
pylab.legend()

name = "swift_background_pdf" 
fname = InspiralUtils.set_figure_name(opts, name)
fname_thumb = InspiralUtils.savefig_pylal(filename=fname, doThumb=True, dpi_thumb=opts.figure_resolution)
fnameList.append(fname)
tagList.append("Background PDF")
pylab.close()

# plot of cumulative number densities

fig_num += 1
pylab.figure(fig_num)
legend_text = []
pylab.plot(swift_bg_x, numpy.cumsum(swift_bg_pdf[::-1])[::-1]*swift_bg_step/swift_bg_pdf_norm, color='k', label="smooth pdf")
pylab.plot(swift_bg_x, numpy.cumsum(swift_bg_power_n[::-1])[::-1]*swift_bg_step/swift_bg_power_norm + right_cum_prob, color='b', linestyle="-", label="power fit")
pylab.plot(swift_bg_bin_centres, numpy.cumsum(swift_bg_n[::-1])[::-1]*swift_bg_bin/swift_bg_hist_norm, "ko", label="bg histogram")
pylab.axvline(x=min_log_flux, color='r', linestyle='--', label="cut-off flux")

pylab.xlabel(r"$\log_{10}(flux \times 10^{-16}),$ erg cm$^{-2}$ s$^{-1}$")
pylab.title("Fig. "+str(fig_num)+": Normalized flux CDF  for serendipitous X-ray sources.")
pylab.legend()

name = "swift_background_cdf" 
fname = InspiralUtils.set_figure_name(opts, name)
fname_thumb = InspiralUtils.savefig_pylal(filename=fname, doThumb=True, dpi_thumb=opts.figure_resolution)
fnameList.append(fname)
tagList.append("Normalized SWIFT background CDF")
pylab.close()



#log plots ###################################

# plot of number denisty for background

fig_num += 1
pylab.figure(fig_num)
legend_text = []
pylab.semilogy(swift_bg_x, swift_bg_power_n, color='b', linestyle="-", label="power fit")
pylab.semilogy(swift_bg_bin_centres, swift_bg_n, "ko", label="histogram")
pylab.axvline(x=min_log_flux, color='r', linestyle='--', label="cut-off flux")

pylab.xlabel(r"$\log_{10}(flux \times 10^{-16}),$ erg cm$^{-2}$ s$^{-1}$")
pylab.title("Fig. "+str(fig_num)+": log-log plot of distribution of flux for serendipitous X-ray sources.")
pylab.legend()

name = "log_swift_background_density" 
fname = InspiralUtils.set_figure_name(opts, name)
fname_thumb = InspiralUtils.savefig_pylal(filename=fname, doThumb=True, dpi_thumb=opts.figure_resolution)
fnameList.append(fname)
tagList.append("SWIFT Background")
pylab.close()

# plot of cumulative number densities

fig_num += 1
pylab.figure(fig_num)
legend_text = []
pylab.semilogy(swift_bg_x, numpy.cumsum(swift_bg_power_n[::-1])[::-1]*swift_bg_step/swift_bg_bin, color='b', linestyle="-", label="power fit")
pylab.semilogy(swift_bg_bin_centres, numpy.cumsum(swift_bg_n[::-1])[::-1], "ko", label="histogram")
pylab.axvline(x=min_log_flux, color='r', linestyle='--', label="cut-off flux")

pylab.xlabel(r"$\log_{10}(flux \times 10^{-16}),$ erg cm$^{-2}$ s$^{-1}$")
pylab.title("Fig. "+str(fig_num)+": log-log plot of cumulative distribution of flux for serendipitous X-ray sources.")
pylab.legend()

name = "log_swift_cum_background_density" 
fname = InspiralUtils.set_figure_name(opts, name)
fname_thumb = InspiralUtils.savefig_pylal(filename=fname, doThumb=True, dpi_thumb=opts.figure_resolution)
fnameList.append(fname)
tagList.append("Cumulative SWIFT background number density")
pylab.close()

#normalized distributions

fig_num += 1
pylab.figure(fig_num)
legend_text = []
pylab.semilogy(swift_bg_x, swift_bg_pdf/swift_bg_pdf_norm, color='k', label="smooth pdf")
pylab.semilogy(swift_bg_x, swift_bg_power_n/swift_bg_power_norm, color='b', linestyle="-", label="power fit")
pylab.semilogy(swift_bg_bin_centres, swift_bg_n/swift_bg_hist_norm, "ko", label="histogram")
pylab.axvline(x=min_log_flux, color='r', linestyle='--', label="cut-off flux")

pylab.xlabel(r"$\log_{10}(flux \times 10^{-16}),$ erg cm$^{-2}$ s$^{-1}$")
pylab.title("Fig. "+str(fig_num)+": log-log plot of normalized flux PDF  for serendipitous X-ray sources.")
pylab.legend()

name = "log_swift_background_pdf" 
fname = InspiralUtils.set_figure_name(opts, name)
fname_thumb = InspiralUtils.savefig_pylal(filename=fname, doThumb=True, dpi_thumb=opts.figure_resolution)
fnameList.append(fname)
tagList.append("Background PDF")
pylab.close()

# plot of cumulative number densities

fig_num += 1
pylab.figure(fig_num)
legend_text = []
pylab.semilogy(swift_bg_x, numpy.cumsum(swift_bg_pdf[::-1])[::-1]*swift_bg_step/swift_bg_pdf_norm, color='k', label="smooth pdf")
pylab.semilogy(swift_bg_x, numpy.cumsum(swift_bg_power_n[::-1])[::-1]*swift_bg_step/swift_bg_power_norm + right_cum_prob, color='b', linestyle="-", label="power fit")
pylab.semilogy(swift_bg_bin_centres, numpy.cumsum(swift_bg_n[::-1])[::-1]*swift_bg_bin/swift_bg_hist_norm, "ko", label="bg histogram")
pylab.axvline(x=min_log_flux, color='r', linestyle='--', label="cut-off flux")

pylab.xlabel(r"$\log_{10}(flux \times 10^{-16}),$ erg cm$^{-2}$ s$^{-1}$")
pylab.title("Fig. "+str(fig_num)+": log-log plot of Normalized flux CDF  for serendipitous X-ray sources.")
pylab.legend()

name = "log_swift_background_cdf" 
fname = InspiralUtils.set_figure_name(opts, name)
fname_thumb = InspiralUtils.savefig_pylal(filename=fname, doThumb=True, dpi_thumb=opts.figure_resolution)
fnameList.append(fname)
tagList.append("Normalized SWIFT background CDF")
pylab.close()





# plot of FAP(rho_e) for swift background

fig_num += 1
pylab.figure(fig_num)
legend_text = []
analytic_curve = pylab.plot(rho_e_array, swift_faps, color='b', linewidth = 2)

bg_em_lrs = [event[0] for event in em_bg_lr_faps]
bg_em_p = numpy.arange(1,len(bg_em_lrs)+1)/float(len(bg_em_lrs))
sample_curve = pylab.plot(numpy.sort(numpy.asarray(bg_em_lrs))[::-1], bg_em_p, color='k')
pylab.fill_between(numpy.sort(numpy.asarray(bg_em_lrs))[::-1], bg_em_p + (bg_em_p*(1.0 - bg_em_p)/float(len(bg_em_lrs)))**0.5, bg_em_p - (bg_em_p*(1.0 - bg_em_p)/float(len(bg_em_lrs)))**0.5, color="gray", alpha = 0.5)

pylab.gca().set_yscale('log')

pylab.xlabel(r"$\rho_e$")
pylab.title("Fig. "+str(fig_num)+r": Normalized background CDF for log of EM likelihood ratio, $\rho_e$.")
pylab.legend((analytic_curve, sample_curve), ("analytic", "bg samples"))

name = "log_lr_swift_background_cdf" 
fname = InspiralUtils.set_figure_name(opts, name)
fname_thumb = InspiralUtils.savefig_pylal(filename=fname, doThumb=True, dpi_thumb=opts.figure_resolution)
fnameList.append(fname)
tagList.append("Normalized SWIFT background CDF for log of LR")
pylab.close()



# plot of FAP(rho_g) for cwb background

cwb_bg_lr_faps

fig_num += 1
pylab.figure(fig_num)
legend_text = []

cwb_bg_lr_faps.sort()
analytic_curve = pylab.plot([event[0] for event in cwb_bg_lr_faps], [event[1] for event in cwb_bg_lr_faps], color='b', linewidth = 2)

bg_gw_lrs = [event[0] for event in cwb_bg_lr_faps]
bg_gw_p = numpy.arange(1,len(bg_gw_lrs)+1)/float(len(bg_gw_lrs))
sample_curve = pylab.plot(numpy.sort(numpy.asarray(bg_gw_lrs))[::-1], numpy.arange(1,len(bg_gw_lrs)+1)/float(len(bg_gw_lrs)), color='k')
pylab.fill_between(numpy.sort(numpy.asarray(bg_gw_lrs))[::-1], bg_gw_p + (bg_gw_p*(1.0 - bg_gw_p)/float(len(bg_gw_lrs)))**0.5, bg_gw_p - (bg_gw_p*(1.0 - bg_gw_p)/float(len(bg_gw_lrs)))**0.5, color="gray", alpha = 0.5)

pylab.gca().set_yscale('log')

pylab.xlabel(r"$\rho_g$")
pylab.title("Fig. "+str(fig_num)+r": Normalized background CDF for log of GW likelihood ratio, $\rho_g$.")
pylab.legend((analytic_curve, sample_curve), ("analytic", "bg samples"))

name = "log_lr_cwb_background_cdf" 
fname = InspiralUtils.set_figure_name(opts, name)
fname_thumb = InspiralUtils.savefig_pylal(filename=fname, doThumb=True, dpi_thumb=opts.figure_resolution)
fnameList.append(fname)
tagList.append("Normalized CWB background CDF for log of LR")
pylab.close()





# plot of joint FAP for background

#joint_bg_trials[trial] = (joint_bg_events, lr_for_joint_events, bg_joint_lr_faps)

fig_num += 1
pylab.figure(fig_num)
legend_text = []
all_bg_joint_lr_faps = []
for trial in range(N_bg_trials):
  all_bg_joint_lr_faps.extend(joint_bg_trials[trial][2])
all_bg_joint_lr_faps.sort()
  
analytic_curve = pylab.semilogy([event[0] for event in all_bg_joint_lr_faps], [event[1] for event in all_bg_joint_lr_faps], color='b', linewidth = 2, label="analytic")
for trial in range(N_bg_trials):
  bg_joint_lrs = [event[0] for event in joint_bg_trials[trial][2]]
  bg_sample_curve=pylab.semilogy(numpy.sort(numpy.asarray(bg_joint_lrs))[::-1], numpy.arange(1,len(bg_joint_lrs)+1)/float(len(bg_joint_lrs)), color='k')

pylab.xlabel(r"$\rho_{joint}$")
pylab.title("Fig. "+str(fig_num)+r": Normalized background CDF for log of joint likelihood ratio, $\rho_{joint} = \rho_e + \rho_g$.")
pylab.legend((analytic_curve, bg_sample_curve), ("analytic", "bg samples"))

name = "log_lr_joint_background_cdf" 
fname = InspiralUtils.set_figure_name(opts, name)
fname_thumb = InspiralUtils.savefig_pylal(filename=fname, doThumb=True, dpi_thumb=opts.figure_resolution)
fnameList.append(fname)
tagList.append("Normalized joint background CDF for log of LR")
pylab.close()





############################# ROC plots ########################################################


fig_num += 1
pylab.figure(fig_num)
legend_text = []
pylab.loglog(all_gw_fap, all_gw_eff, color='k', linestyle="--", linewidth=2, marker ="o", markersize=0.0, label="CWB")
pylab.loglog(localized_joint_fap, localized_joint_eff, color='b', linestyle="-", linewidth=2, marker ="s", markersize=0.0,label="CWB+SWIFT, localized only")
pylab.loglog(all_joint_fap, all_joint_eff, color='r', linestyle="-", linewidth=2, marker="^", markersize=0.0,label="CWB+SWIFT, all injections")
pylab.loglog(hybrid_joint_fap, hybrid_joint_eff, color='g', linestyle="-", linewidth=2, marker="d", markersize=0.0, label="CWB+SWIFT, realistic")

pylab.axvline(1.0 - scipy.special.erf(1.0/2.0**0.5), color="gray", linestyle="--")
pylab.axvline(1.0 - scipy.special.erf(2.0/2.0**0.5), color="gray", linestyle="--")
pylab.axvline(1.0 - scipy.special.erf(3.0/2.0**0.5), color="gray", linestyle="--")
pylab.axvline(1.0 - scipy.special.erf(4.0/2.0**0.5), color="gray", linestyle="--")
pylab.axvline(1.0 - scipy.special.erf(5.0/2.0**0.5), color="gray", linestyle="--")
pylab.axvline(1.0 - scipy.special.erf(6.0/2.0**0.5), color="gray", linestyle="--")
pylab.axvline(1.0 - scipy.special.erf(7.0/2.0**0.5), color="gray", linestyle="--")
pylab.axvline(1.0 - scipy.special.erf(8.0/2.0**0.5), color="gray", linestyle="--")
pylab.axvline(1.0 - scipy.special.erf(9.0/2.0**0.5), color="gray", linestyle="--")
pylab.axvline(1.0 - scipy.special.erf(10.0/2.0**0.5), color="gray", linestyle="--")

pylab.xlabel(r"False Alarm Probability")
pylab.ylabel(r"Efficiency")
pylab.title("Fig. "+str(fig_num)+": ROC curves for CWB and CWB+SWIFT searches")
pylab.legend(loc=2)

name = "roc_curves" 
fname = InspiralUtils.set_figure_name(opts, name)
fname_thumb = InspiralUtils.savefig_pylal(filename=fname, doThumb=True, dpi_thumb=opts.figure_resolution)
fnameList.append(fname)
tagList.append("ROC curves for CWB and CWB+SWIFT searches")
pylab.close()



####################### Diagonstic plots for injections ############################################

# make a few plots for original set of injections with eta >= 3.0


fig_num += 1
pylab.figure(fig_num)
legend_text = []

pylab.semilogy(original_injs_distance, original_injs_stat, 'xb')
pylab.xlabel("distance, Mpc")
pylab.ylabel(r"$eta$")
pylab.title("Fig. "+str(fig_num)+r":Injections with $\eta \geq 3.0$, distance vs EM flux")
pylab.legend()

name = "original_injections_distance_vs_stat"
fname = InspiralUtils.set_figure_name(opts, name)
fname_thumb = InspiralUtils.savefig_pylal(filename=fname, doThumb=True, dpi_thumb=opts.figure_resolution)
fnameList.append(fname)
tagList.append(name)
pylab.close()


fig_num += 1
pylab.figure(fig_num)
legend_text = []

pylab.hist(original_injs_distance, bins=10)
pylab.xlabel("distance, Mpc")
#pylab.ylabel(r"$eta$")
pylab.title("Fig. "+str(fig_num)+": Histogram of injections with $\eta \geq 3.0$,")
pylab.legend()

name = "original_injections_distance_hist"
fname = InspiralUtils.set_figure_name(opts, name)
fname_thumb = InspiralUtils.savefig_pylal(filename=fname, doThumb=True, dpi_thumb=opts.figure_resolution)
fnameList.append(fname)
tagList.append(name)
pylab.close()





inj_above_gw_lr_threshold = all_injections.get_above_gw_lr_threshold(1.0)
tmp_above_gw_lr_threshold = tmp_injections.get_above_gw_lr_threshold(1.0)

fig_num += 1
pylab.figure(fig_num)
legend_text = []

pylab.plot(inj_above_gw_lr_threshold.get_column("distance"), inj_above_gw_lr_threshold.get_column("em_flux"), 'xr')
pylab.xlabel("distance, Mpc")
pylab.ylabel(r"$\log_{10}(flux \times 10^{-16}),$ erg cm$^{-2}$ s$^{-1}$")
pylab.title("Fig. "+str(fig_num)+": Injections, distance vs EM flux")
pylab.legend()

name = "injections_distance_vs_em_flux"
fname = InspiralUtils.set_figure_name(opts, name)
fname_thumb = InspiralUtils.savefig_pylal(filename=fname, doThumb=True, dpi_thumb=opts.figure_resolution)
fnameList.append(fname)
tagList.append(name)
pylab.close()

fig_num += 1
pylab.plot(inj_above_gw_lr_threshold.get_column("distance"), inj_above_gw_lr_threshold.get_column("gw_lr"), 'xr')
pylab.xlabel("distance, Mpc")
pylab.ylabel(r"$\rho_g$")
pylab.title(r"Fig. "+str(fig_num)+r": Injections, distance vs $\rho_g$")
pylab.legend()

name = "injections_distance_vs_gw_lr"
fname = InspiralUtils.set_figure_name(opts, name)
fname_thumb = InspiralUtils.savefig_pylal(filename=fname, doThumb=True, dpi_thumb=opts.figure_resolution)
fnameList.append(fname)
tagList.append(name)
pylab.close()



fig_num += 1
pylab.semilogx(inj_above_gw_lr_threshold.get_column("gw_stat"), inj_above_gw_lr_threshold.get_column("gw_lr"), 'xr')
pylab.xlabel(r"$\eta$")
pylab.ylabel(r"$\rho_g$")
pylab.title(r"Fig. "+str(fig_num)+r": Injections,  $\eta$ vs $\rho_g$")
pylab.legend()

name = "injections_rho_vs_gw_lr"
fname = InspiralUtils.set_figure_name(opts, name)
fname_thumb = InspiralUtils.savefig_pylal(filename=fname, doThumb=True, dpi_thumb=opts.figure_resolution)
fnameList.append(fname)
tagList.append(name)
pylab.close()




fig_num += 1
pylab.plot(inj_above_gw_lr_threshold.get_column("distance"), inj_above_gw_lr_threshold.get_column("em_lr"), 'xr')
pylab.xlabel("distance, Mpc")
pylab.ylabel(r"$\rho_e$")
pylab.title(r"Fig. "+str(fig_num)+r": Injections, distance vs $\rho_e$.")
pylab.legend()


name = "injections_distance_vs_em_lr"
fname = InspiralUtils.set_figure_name(opts, name)
fname_thumb = InspiralUtils.savefig_pylal(filename=fname, doThumb=True, dpi_thumb=opts.figure_resolution)
fnameList.append(fname)
tagList.append(name)
pylab.close()

fig_num += 1
pylab.plot(inj_above_gw_lr_threshold.get_column("distance"), inj_above_gw_lr_threshold.get_column("joint_lr"), 'xr')
pylab.xlabel("distance, Mpc")
pylab.ylabel(r"\rho_{joint}$")
pylab.title(r"Fig. "+str(fig_num)+r": Injections, distance vs $\rho_{joint}$.")
pylab.legend()
name = "injections_distance_vs_joint_lr"
fname = InspiralUtils.set_figure_name(opts, name)
fname_thumb = InspiralUtils.savefig_pylal(filename=fname, doThumb=True, dpi_thumb=opts.figure_resolution)
fnameList.append(fname)
tagList.append(name)
pylab.close()

fig_num += 1
pylab.semilogy(inj_above_gw_lr_threshold.get_column("gw_lr"), inj_above_gw_lr_threshold.get_column("gw_fap"), 'xr')
pylab.xlabel(r"$\rho_g$")
pylab.ylabel(r"FAP($\rho_g$)")
pylab.title(r"Injections: $\rho_g$ vs FAP($\rho_g$)")
pylab.legend()

name = "injections_gw_lr_vs_gw_fap"
fname = InspiralUtils.set_figure_name(opts, name)
fname_thumb = InspiralUtils.savefig_pylal(filename=fname, doThumb=True, dpi_thumb=opts.figure_resolution)
fnameList.append(fname)
tagList.append(name)
pylab.close()

fig_num += 1
pylab.semilogy(inj_above_gw_lr_threshold.get_column("em_lr"), inj_above_gw_lr_threshold.get_column("em_fap"), 'xr')
pylab.xlabel(r"$\rho_e$")
pylab.ylabel(r"FAP($\rho_e$)")
pylab.title(r"Fig. "+str(fig_num)+r": Injections,  $\rho_e$ vs FAP($\rho_e$)")
pylab.legend()

name = "injections_em_lr_vs_em_fap"
fname = InspiralUtils.set_figure_name(opts, name)
fname_thumb = InspiralUtils.savefig_pylal(filename=fname, doThumb=True, dpi_thumb=opts.figure_resolution)
fnameList.append(fname)
tagList.append(name)
pylab.close()

fig_num += 1
pylab.semilogy(inj_above_gw_lr_threshold.get_column("joint_lr"), inj_above_gw_lr_threshold.get_column("joint_fap"), 'xr')
pylab.xlabel(r"$\rho_{joint}$")
pylab.ylabel(r"FAP($\rho_{joint}$)")
pylab.title(r"Fig. "+str(fig_num)+r": Injections, $\rho_{joint}$ vs FAP($\rho_{joint}$)")
pylab.legend()

name = "injections_djoint_lr_vs_joint_fap"
fname = InspiralUtils.set_figure_name(opts, name)
fname_thumb = InspiralUtils.savefig_pylal(filename=fname, doThumb=True, dpi_thumb=opts.figure_resolution)
fnameList.append(fname)
tagList.append(name)
pylab.close()



fig_num += 1
sigmas = []
xticks_labels = []
for i in range(8,0,-1):
  sigmas.append(1.0 - scipy.special.erf(float(i)/2.0**0.5))
  xticks_labels.append(str(i)+r"$\sigma$")

pylab.loglog(inj_above_gw_lr_threshold.get_column("gw_fap"), inj_above_gw_lr_threshold.get_column("joint_fap"), 'xr', label="all injections")
pylab.loglog(tmp_above_gw_lr_threshold.get_column("gw_fap"), tmp_above_gw_lr_threshold.get_column("joint_fap"), '+g', label="observed injections")
pylab.loglog([sigmas[0], sigmas[-1]], [sigmas[0], sigmas[-1]], linestyle="--")
pylab.xlabel(r"FAP($\rho_g$")
pylab.ylabel(r"FAP($\rho_{joint}$)")
pylab.title(r"Fig. "+str(fig_num)+r": Injections,  FAP($\rho_{joint}$) vs FAP($\rho_g$)")
pylab.legend(loc=2)

pylab.xticks(sigmas, xticks_labels)
pylab.yticks(sigmas, xticks_labels)
pylab.xlim(xmin=(1.0 - scipy.special.erf(8.0/2.0**0.5)))
pylab.ylim(ymin=(1.0 - scipy.special.erf(8.0/2.0**0.5)))
pylab.grid()


name = "injections_joint_fap_vs_gw_fap"
fname = InspiralUtils.set_figure_name(opts, name)
fname_thumb = InspiralUtils.savefig_pylal(filename=fname, doThumb=True, dpi_thumb=opts.figure_resolution)
fnameList.append(fname)
tagList.append(name)
pylab.close()




#joint_bg_trials[trial] = (joint_bg_events, lr_for_joint_events, bg_joint_lr_faps)

fig_num += 1
pylab.figure(fig_num)
legend_text = []
all_bg_joint_lrs = []
for trial in range(N_bg_trials):
  all_bg_joint_lrs.extend(joint_bg_trials[trial][1])



pylab.plot(inj_above_gw_lr_threshold.get_column("gw_lr"), inj_above_gw_lr_threshold.get_column("em_lr"), 'xr', label="all injections")
pylab.plot(tmp_above_gw_lr_threshold.get_column("gw_lr"), tmp_above_gw_lr_threshold.get_column("em_lr"), '+g', label="realistic injections")
pylab.plot([event[0] for event in all_bg_joint_lrs], [event[1] for event in all_bg_joint_lrs], 'xk', label = "background")
pylab.xlabel(r"$\rho_g$")
pylab.ylabel(r"$\rho_e$")
pylab.title(r"Fig. "+str(fig_num)+r": $\rho_g$ vs $\rho_e$ for joint events")
pylab.legend(loc=2)
pylab.ylim(ymax=100.0)
pylab.xlim(xmax=100.0)
name = "gw_em_lr"
fname = InspiralUtils.set_figure_name(opts, name)
fname_thumb = InspiralUtils.savefig_pylal(filename=fname, doThumb=True, dpi_thumb=opts.figure_resolution)
fnameList.append(fname)
tagList.append(name)
pylab.close()



##############################################################################################################


html_filename = InspiralUtils.write_html_output(opts, args, fnameList, tagList, comment=comments)
InspiralUtils.write_cache_output(opts, html_filename, fnameList)

if opts.html_for_cbcweb:
  html_filename_publish = InspiralUtils.wrifacte_html_output(opts, args, fnameList, tagList, cbcweb=True)

##############################################################################################################


 





