from glue.ligolw import table
from glue.ligolw import lsctables
from glue.ligolw import utils
import numpy
import model_fit
import scipy.integrate

# Table to store injections

class GWEMInjectionTable(table.Table):
  tableName = "gwem_injection:table"
  validcolumns = {
    "gps_time_s": "int_4s",
    "gps_time_ns": "int_4s",
    "distance": "real_4",
    "hrss_fac": "real_4",
    "type": "lstring",
    "gw_stat": "real_4",
    "em_flux": "real_4",
    "ra": "real_4",
    "dec": "real_4",
    "in_top_5_swift_pxls": "int",
    "sky_map_prob": "real_4",
    "prob_in_top_5_swift_pxls": "real_4",
    "gw_lr": "real_4",
    "em_lr": "real_4",
    "joint_lr": "real_4",
    "gw_fap": "real_4",
    "em_fap": "real_4",
    "joint_fap": "real_4",
    "event_id": "int_8s"
  }
  def get_column(self,column):
    if column == "type":
      return self.getColumnByName(column)
    else:
      return numpy.asarray(self.getColumnByName(column))

  def get_subclass(self, type, hrss_fac):
    subclass = lsctables.New(GWEMInjectionTable)
    for row in self:
      if (row.type == type) and (row.hrss_fac == hrss_fac):
        subclass.append(row)
    return subclass
    
  def get_localized(self):
    localized = lsctables.New(GWEMInjectionTable)
    for row in self:
      if row.in_top_5_swift_pxls == 1:
        localized.append(row)
    return localized
  def get_above_gw_lr_threshold(self,threshold):
    """
    Returns injection whose gw_lr is above threshold.
    """
    above_threshold = lsctables.New(GWEMInjectionTable)
    for row in self:
      if row.gw_lr >= threshold:
        above_threshold.append(row)
        
    return above_threshold
    
  def populate_em_flux_column(self, model_flux, model_distance):
    """
    Populate em_flux column using the model flux and scale it by distance for each injection.
    """
  
    for row in self:
      row.em_flux = numpy.log10((10.0**model_flux) * model_distance**2 / row.distance**2)
    
  def calculate_gw_lr(self, bg_exppowpar, inj_curve, LR_scale):
    """
    Calculate gw likelihood ratio for each injection.
    """
      
    for row in self:
      row.gw_lr = numpy.log(LR_scale * calculate_cwb_lr(row.gw_stat, bg_exppowpar, inj_curve))  
    
  def calculate_em_lr(self, par):
    """
    Calculate swift part of likelihood ratio for each injection.
    """  
    for row in self:
      row.em_lr = numpy.log(calculate_swift_lr(row.em_flux, par))
      
  def calculate_joint_lr(self):
    """
    Calculate joint GW+EM likelihood ratio for each injection.
    It is defined as sum of logs of gw and em likelihood ratios.
    """
    for row in self:
      row.joint_lr = row.gw_lr + row.em_lr    
      
  def calculate_gw_fap(self, cwb_par):
    """
    Calculates gw FAP for each injection using cwb likelihood ratio and analytic background curve for cwb.
    """
    for row in self:
      row.gw_fap = calculate_cwb_fap(row.gw_lr, cwb_par)
      
  def calculate_em_fap(self, swift_par, x_min):
    """
    Calculates em FAP for each injection using swift background curve.
    """    
    for row in self:
      (FAP, x) = SWIFT_FAP(row.em_lr, swift_par, x_min)
      row.em_fap = FAP
      
  def calculate_joint_fap(self, swift_par, cwb_par, rho_e_min, rho_g_min, min_flux):
    """
    Calculates joing FAP by integrating em and gw faps.
    """
    for row in self:  
      row.joint_fap = CWB_SWIFT_FAP(row.joint_lr, swift_par, cwb_par, rho_e_min, rho_g_min, min_flux)
      
class GWEMInjection(object):
  __slots__ = GWEMInjectionTable.validcolumns.keys()


GWEMInjectionTable.RowType = GWEMInjection


def read_gwem_injections(file):
  """
  Reads GWEMInjectionTable from xml file
  """

  xmldoc = utils.load_filename(file, gz=False, verbose=False)
  xmltable = table.get_table(xmldoc, GWEMInjectionTable.tableName)
  injections = lsctables.New(GWEMInjectionTable)
  
  
  elems = xmltable[0].__dict__.keys()
  for row in xmltable:
    inj_row = GWEMInjection()
    for elem in elems:
      setattr(inj_row, elem, getattr(row, elem))
    injections.append(inj_row)  
  
  return injections


def calculate_cwb_lr(rho, bg_exppowpar, inj_curve):
  """
  Calculates likelihhod ratio for cWb triggers.
  """
  
  sig_prob = float(inj_curve(numpy.log10(rho)))
  bg_prob = model_fit.exppow(bg_exppowpar, rho)
  
  lr = sig_prob/bg_prob
  
  return lr
  
  
def calculate_swift_lr(x, par):
  """
  Calculates swift likelihood ratio defined as constant/p_e(x), where
  p_e(x) is pdf to observe serendipitous source with flux x.
  """  
  (alpha1, S0 , K, stat_scale) = (abs(par[0]), abs(par[1]), abs(par[2]), abs(par[3]))
  
  return stat_scale / SWIFT_bg_pdf(x, alpha1, S0, K) 
  

def calculate_cwb_fap(rho_g, cwb_par):
  """
  Calculates FAP - probability of observing a background event with rho_g'>= rho_g.
  rho_q is logarith of cwb likelihood ratio.
  It uses analytic background distribution for rho_g: P(rho_g) = exp(-b*rgo_g) / (b*N).
  """
  (b, N)  = (abs(cwb_par[0]), abs(cwb_par[1]))
  
  return numpy.exp(-b*rho_g) / (b*N)


def cwb_fap_pdf(rho_g, cwb_par):  
  """
  Calculates pdf of FAP - probability of observing a background event with rho_g'>= rho_g.
  rho_q is logarith of cwb likelihood ratio.
  It uses analytic background distribution for rho_g: P(rho_g) = exp(-b*rgo_g) / (b*N).
  """
  (b, N)  = (abs(cwb_par[0]), abs(cwb_par[1]))
  
  return numpy.exp(-b*rho_g) / N


  
def read_swift_catalog(file):
  """
  Reads in catalog of SWIFT serendipitous sources.
  """

  swift_file = open(file, "r")
  fluxes = []
  
  for (i, line) in enumerate(swift_file):
  
    if i > 0:
      flux = line.split("|")[2]
      if "<" in flux:
        flux = float(flux[3:-1])
      else:
        flux = float(flux[1:-1])
      fluxes.append(flux)
      
  return numpy.asarray(fluxes)
  
def read_xmm_targets(XMM_targets_catalog):
  """
  Read in sky-location of detected XMM targets from observations used in 2XMMi-DR3 catalog of serendipitous soruces. 
  """
  XMM_targets = []
  # read in list of XMM targets
  xmm_targets_file = open(XMM_targets_catalog, "r")
  for (i,line) in enumerate(xmm_targets_file):
    if i > 0:
      line_list = line.split(",")
      entries = []
      for entry in line_list:
        if not entry == "  ":
          entries.append(entry)
      # get  RA, DEC of the source unless it is non-detection
      if not entries[2] == "NULL ":
        XMM_targets.append((float(entries[2]), 0.0, float(entries[3]), 0.0))
  return XMM_targets

def read_rass_sources(RASS_catalog):
  """
  Read the list of locations of X-ray sources from RASS catalog.
  """
  
  rass_sources = []
  
  rass_file = open(RASS_catalog, "r")
  
  for (i,line) in enumerate(rass_file):
    if i > 3:
      ra = float(line[23:32])
      dec = float(line[33:42])
      # position error is measured in arcseconds, converting it into degrees 
      err = float(line[44:46]) / 3600.0
      ext = float(line[118:123])
      # keep only point like sources, threshol is set by XMM limit of 12 arcsec in diameter
      if ext <= 12:
        rass_sources.append((ra, err, dec, err))
  
  return rass_sources
        
def distance_btw_sources(s1,s2):
  """
  Computes statistical measure of separation between two source locations. It reflects their odds to correspond to the same source.
  It return r = (\delta RA^2 / ss_sigma_ra^2 + \delta DEC ^2 / ss_sigma_dec^2)^1/2, where ss_sigma_ra^2 = sigma_ra_(s1)^2 + sigma_ra(s2)^2 is the total sigma squared for both locations. 
  ss_sigma_dec^2 is defined similarly. \delta RA and \delta DEC are differences in RA and DEC of the sourc locations, s1 and s2.
  r < 2.146, 3.035,3.439 correspond to 1,2 and 3 sigma probability that source locations correspond to single source. 
  Reference: M. G. Watson et al A&A 493, 339-373 (2009) DOI: 10.1051/0004-6361:200810534 
  s1,s2 = (RA, sigma_RA, DEC, sigma_DEC)
  """
  delta_RA_sq = min(abs(s1[0] - s2[0]), 360.0 - abs(s1[0] - s2[0]))**2 
  delta_DEC_sq = (s1[2] - s2[2])**2
  ss_sigma_RA = s1[1]**2 + s2[1]**2
  ss_sigma_DEC = s1[3]**2 + s2[3]**2 
  return (delta_RA_sq / ss_sigma_RA + delta_DEC_sq / ss_sigma_DEC)**0.5

def filter(sources, locs, sigmas=3):
  """
  Get rid of sources that are within sigmas of any of the locations given in locs list.
  """
  
  # set threshold in proximity
  if sigmas == 3:
    r = 3.439
  elif sigmas == 2:
    r = 3.035
  elif sigmas == 1:
    r = 2.146
  else:
    print "Wrong input for sigmas. sigmas must be either 1, 2 or 3."
    sys.exit(1)
  
  
  
  filtered_sources = []
  
  for source in sources:
    near = False
    for loc in locs:
      if distance_btw_sources(source[1], loc) <= r:
        near = True
    if not near:
      filtered_sources.append(source)
    
  return filtered_sources
  
  

  
def read_xmmi_catalog(data_file_name, format_file_name, XMM_targets = None, RASS_catalog = None, quality_flag = 0, variable_only=False):
  """
  Reads 2XMMi-DR3 catalog of X-ray sources.
  """ 
  # construct mapping between column names and column index in the data file
  column_index = {}
  
  format_file = open(format_file_name, "r")
  
  for (i, line) in enumerate(format_file):
    if i > 0:
      entries = line.split(" ")
      column_index[entries[0]] = int(entries[-1])
  format_file.close()    

    
  catalog_file = open(data_file_name, "r")
  lgl_fluxes = {}
  hgl_fluxes = {}
  
  for line in catalog_file:
    fields = line.split(",")
    SRCID = int(fields[column_index["SRCID"] - 1])
    EP_8_FLUX = float(fields[column_index["EP_8_FLUX"] - 1])
    EP_8_FLUX_ERR = float(fields[column_index["EP_8_FLUX_ERR"] - 1])
    EP_EXTENT = float(fields[column_index["EP_EXTENT"] - 1])
    SUM_FLAG = int(fields[column_index["SUM_FLAG"] - 1])
    VAR_FLAG = fields[column_index["VAR_FLAG"] - 1]
    BII = float(fields[column_index["BII"] - 1])
    RA = float(fields[column_index["RA"] - 1])
    DEC = float(fields[column_index["DEC"] - 1])
    #read positional error ( in arcseconds) and converted it to degrees
    POSERR = (float(fields[column_index["POSERR"] - 1])) / 3600.00
    
    #impose quality flag
    if SUM_FLAG <= quality_flag:
      # allow only point sources:
      if EP_EXTENT == 0.0:
        # check if the source at low or high galactic latitude
        if abs(BII) > 20.0:
          # check for variability
          if variable_only:
            if VAR_FLAG == "TRUE":
              # check if observation corresponds to a new source
              if SRCID in hgl_fluxes.keys():
                hgl_fluxes[SRCID].append((EP_8_FLUX, EP_8_FLUX_ERR, RA, DEC, POSERR))
              else:
                hgl_fluxes[SRCID] = [(EP_8_FLUX, EP_8_FLUX_ERR, RA, DEC, POSERR)]
            
          else:
            # check if observation corresponds to a new source
            if SRCID in hgl_fluxes.keys():
              hgl_fluxes[SRCID].append((EP_8_FLUX, EP_8_FLUX_ERR, RA, DEC, POSERR))
            else:
              hgl_fluxes[SRCID] = [(EP_8_FLUX, EP_8_FLUX_ERR, RA, DEC, POSERR)]
        
        else:
        
          # check for variability
          if variable_only:
            if VAR_FLAG == "TRUE":
              # check if observation corresponds to a new source
              if SRCID in lgl_fluxes.keys():
                lgl_fluxes[SRCID].append((EP_8_FLUX, EP_8_FLUX_ERR, RA, DEC, POSERR))
              else:
                lgl_fluxes[SRCID] = [(EP_8_FLUX, EP_8_FLUX_ERR, RA, DEC, POSERR)]
            
          else:
            # check if observation corresponds to a new source
            if SRCID in lgl_fluxes.keys():
              lgl_fluxes[SRCID].append((EP_8_FLUX, EP_8_FLUX_ERR, RA, DEC, POSERR))
            else:
              lgl_fluxes[SRCID] = [(EP_8_FLUX, EP_8_FLUX_ERR, RA, DEC, POSERR)]
          
        
  catalog_file.close()
  
  
  # compute mean flux, ra, dec end poserr weighted by inverse variance  for each unique source                                        
  lgl_source_fluxes = []
  hgl_source_fluxes = []
  
  for key in hgl_fluxes.keys():
    mean_flux = 0.0
    norm_flux = 0.0
    mean_dec = 0.0
    norm_dec = 0.0
    mean_ra = 0.0
    norm_ra = 0.0
    mean_poserr = 0.0
    for (flux, sigma_flux, ra, dec, poserr) in hgl_fluxes[key]:
      mean_flux += flux / sigma_flux**2
      norm_flux += 1.0 / sigma_flux**2
      mean_ra += ra / poserr**2
      norm_ra += 1.0 / poserr**2
      mean_dec += dec / poserr**2
      norm_dec += 1.0 / poserr**2
      mean_poserr += 1.0/poserr**2
      
    mean_flux = mean_flux / norm_flux
    mean_ra = mean_ra / norm_ra 
    mean_dec = mean_dec / norm_dec
    mean_poserr = (1.0/ mean_poserr)**0.5
    s_loc = (mean_ra, mean_poserr, mean_dec, mean_poserr)   
    hgl_source_fluxes.append((mean_flux, s_loc))
    
    
  for key in lgl_fluxes.keys():
    mean_flux = 0.0
    norm_flux = 0.0
    mean_dec = 0.0
    norm_dec = 0.0
    mean_ra = 0.0
    norm_ra = 0.0
    mean_poserr = 0.0
    for (flux, sigma_flux, ra, dec, poserr) in lgl_fluxes[key]:
      mean_flux += flux / sigma_flux**2
      norm_flux += 1.0 / sigma_flux**2
      mean_ra += ra / poserr**2
      norm_ra += 1.0 / poserr**2
      mean_dec += dec / poserr**2
      norm_dec += 1.0 / poserr**2
      mean_poserr += 1.0/poserr**2
      
    mean_flux = mean_flux / norm_flux
    mean_ra = mean_ra / norm_ra 
    mean_dec = mean_dec / norm_dec
    mean_poserr = (1.0/ mean_poserr)**0.5
    s_loc = (mean_ra, mean_poserr, mean_dec, mean_poserr)   
    lgl_source_fluxes.append((mean_flux, s_loc))
    
  # fliter out target sources  
  # read in list of XMM detected targets
  if XMM_targets:
    XMM_targets = read_xmm_targets(XMM_targets)
    hgl_source_fluxes = filter(hgl_source_fluxes,XMM_targets)
    lgl_source_fluxes = filter(lgl_source_fluxes,XMM_targets) 
    
   
  # read in location of sources from RASS catalog
  if RASS_catalog:
    RASS_sources = read_rass_sources(RASS_catalog)
    hgl_source_fluxes = filter(hgl_source_fluxes,RASS_sources)
    lgl_source_fluxes = filter(lgl_source_fluxes,RASS_sources) 
         
             
  return hgl_source_fluxes, lgl_source_fluxes
  
  
def swift_flux_stat_func(x, par, rho_e, x_min):

  (alpha1, S0 , K, stat_scale) = (abs(par[0]), abs(par[1]), abs(par[2]), abs(par[3]))
  if x > x_min:
    f = SWIFT_bg_pdf(x, alpha1, S0, K) - stat_scale*numpy.exp(-rho_e)
  else:
    f = numpy.inf
  return f
    
  
  
        
def cwb_swift_comb_fap_integrant(x, swift_par, cwb_par):
  """
  Integrant for estimation of FAP for combined CWB-SWIFT statistic.
  """
  
  (alpha1, S0 , K, stat_scale) = (abs(swift_par[0]), abs(swift_par[1]), abs(swift_par[2]), abs(swift_par[3]))
  (b, N)  = (abs(cwb_par[0]), abs(cwb_par[1])) 
  
  S = 10.0**x
  dS = 10.0**x*numpy.log(10.0)
  drho_e = ((alpha1 + 1.0)*S**alpha1 - (alpha1 - 1.0)*S0**alpha1) / (S*(S**alpha1 + S0**alpha1))
  prho_e = (K*alpha1*S**(alpha1 - 1.0)) / (S**alpha1 + S0**alpha1)**2.0
  pg_rho_e = ( (stat_scale*(S**alpha1 + S0**alpha1)**2.0 ) / (K*alpha1*S**(alpha1 - 1.0)) )**(b)
  
  integrant = drho_e * prho_e * pg_rho_e * dS
  #integrant = (K*alpha1)**(1 - b)*(10.0**(x*((alpha1-1)*(b-1)-1)))*((alpha1 + 1)*10.0**(x*alpha1) - (alpha1 -1) * S0**alpha1)*(10.0**(x*alpha1) + S0**alpha1)**(2*b-3)
  
  return integrant
  

def SWIFT_FAP(rho_e, par, x_min):
  """
  Returns value of SWIFT FAP for rho_e, where rho_e = ln(C/p(s)), s is X-ray flux, C is a constant.
  """  
  #init_x = 1.3*x_min
  init_x = 1.2*x_min
  x = scipy.optimize.fsolve(swift_flux_stat_func, init_x, args = (par, rho_e, x_min))
  
  (alpha1, S0 , K) = (abs(par[0]), abs(par[1]), abs(par[2]))
  
  FAP = K/(10.0**(x*alpha1) + S0**alpha1)
  
  return (FAP,x) 
  

def joint_fap_integrant(x, upper_lim, min_flux, swift_par, cwb_par):
  """
  p_{cwb}(x)*P_{swift}(upper_limit - x)
  """
  
  return cwb_fap_pdf(x, cwb_par) * SWIFT_FAP(upper_lim - x, swift_par, min_flux)[0]
  
def CWB_SWIFT_FAP(x, swift_par, cwb_par, rho_e_min, rho_g_min, min_flux):
  """
  Return false alarm probability for combined CWB-SWIFT statistic.
  """  
  
  
  #init_x = 1.2*(min_flux -0.5)
  #lower_lim = min_flux
  #upper_lim = scipy.optimize.fsolve(swift_flux_stat_func, init_x, args = (swift_par, x - rho_g_min, min_flux-0.5))
 
  #integral = scipy.integrate.quad(cwb_swift_comb_fap_integrant, lower_lim, upper_lim, args=(swift_par, cwb_par))
   
  #print integral
  #(b, N)  = (abs(cwb_par[0]), abs(cwb_par[1]))
  #FAP = (1.0/ (N*b))*numpy.exp(-b*x)*integral[0] + SWIFT_FAP(x-rho_g_min, swift_par, min_flux - 0.5)[0]
  lower_lim = rho_g_min
  upper_lim = x - rho_g_min
  integral = scipy.integrate.quad(joint_fap_integrant, lower_lim, upper_lim, args=(upper_lim+rho_e_min, min_flux, swift_par, cwb_par))
    
   
  FAP = integral[0] + calculate_cwb_fap(upper_lim, cwb_par)
   
  return FAP
  
        
def SWIFT_cum_dist(x, par):
  """
  Analytic cumulative distirbution for swift background.
  x is log_10(flux).
  """  
  (alpha1, S0 , K) = (abs(par[0]), abs(par[1]), abs(par[2]))
  
  return K/(10.0**(x*alpha1) + S0**alpha1)
   
   
def SWIFT_cum_dist_opt_func(x, par, N):   
  
  return SWIFT_cum_dist(x, par) - N
  
  
def SWIFT_bg_pdf(x, alpha1, S0, K):
  """
  Analytical PDF for swift serendipitous sources.
  """ 
  return K*(numpy.log(10.0)*alpha1*10.0**(x*alpha1))/ (10.0**(x*alpha1) + S0**alpha1)**2  
  
  
def ROC(injections, fap_column_name, localized_only = True):
  """
  Constructs ROC curve for injections using on of the three faps: gw_fap, em_fap or joint_fap.
  """  
  norm = len(injections)
  
  # get injections with gw_lr > 1
  # gw_lr =1 corresponds to threshold rho for background, rho = 3.5
  inj_above_threshold = injections.get_above_gw_lr_threshold(1.0)
  
  if localized_only:
    roc_injections = inj_above_threshold.get_localized()
  else:
    roc_injections = inj_above_threshold
    
  faps = roc_injections.get_column(fap_column_name)
  faps.sort()
  effs = numpy.cumsum(numpy.ones(len(faps))) / float(norm)
  
  return (faps, effs)
  
  
          
