# this file has definitions used for the likelihood fits

import scipy.special
from scipy.optimize import fmin
import numpy




# return goodness of fit for data y(x) to function fun(par)

# sum squared fractional deviation
# take max(u, mu) instead of fitting in log space.
def fractionalsumsq(par, fun, x, y):
	mu = fun(par, x)
	return sum(((y-mu) / numpy.amax(numpy.vstack((y, mu)), axis=0))**2)

# chi-sq in log space
def logchisq(par, fun, x, y):
	mu = fun(par, x)
	return sum((numpy.log(mu/y))**2)

# binned negative log likelihood based on poisson statistics
def negloglikelihood(par, fun, x, y):
    mu = fun(par, x)
    return -sum(y * numpy.log(mu) - mu - scipy.special.gammaln(y+1))
    
 
# binned negative log likelihood based on poisson statistics with bin-dependent weights
# contribution to loglikelihhod from each bin is scaled by w(bin)
def weightednegloglikelihood(par, fun, x, y, w):
	mu = fun(par, x)
	return -sum(w * (y * numpy.log(mu) - mu - scipy.special.gammaln(y+1)))

# binned negative log likelihood based on poisson statistics assuming weighted events
# bin measurement is assumed to be the (non-integer) sum of events weighted by w(bin)
# (mu, y) -> (mu/w, y/w) for the calculation of poisson statistics
def eventweightednegloglikelihood(par, fun, x, y, w):
	mu = fun(par, x)
	return -sum(y/w * numpy.log(mu/w) - mu/w - scipy.special.gammaln(y/w+1))

# fractional sum-sq above n events, possion statistics after fewer than n events per bin
# fractional sum-sq will have correct statistics at n events to match poisson likelihood
def hybridfitold(par, fun, x, y, n):
    mu = fun(par, x)
    fssq = (y >= n) * n * ((y-mu) / numpy.amax(numpy.vstack((y, mu)), axis=0))**2
    nll  = (y <  n) * -(y * numpy.log(mu) - mu - scipy.special.gammaln(y+1))
    return(sum(fssq + nll))

# fixed hybrid fit
def hybridfit(par, fun, x, y, n):
    mu = fun(par, x)
    fssq = (y >= n) * ((n/2.0)*((y-mu)/numpy.amax(numpy.vstack((y, mu)), axis=0))**2 + 0.5*numpy.log(2*numpy.pi*n))
    # nlgaussian  = (y <  n) * (log(2 * pi * mu)/2.0 + (y - mu)**2 / (2 * mu))
    nll  = (y <  n) * -(y * numpy.log(mu) - mu - scipy.special.gammaln(y+1))
    return(sum(fssq + nll))

# power law shaped by 2nd degree polynomial
def powpoly(par, x):
    (a, b, c, d) = (par[0], par[1], par[2], par[3])
    mu = x ** a * (b + c * x + d * x**2)
    return mu

# straight line
def linear(par, x):
    (a, b) = (par[0], par[1])
    mu = a + b * x
    return mu

# second order polynomail
def poly(par, x):
    (a, b, c) = (par[0], par[1], par[2])
    mu = a + b * x + c * x**2
    return mu

# power law
def powerlaw(par, x):
    (a, b) = (abs(par[0]), abs(par[1]))
    try: 
      mu = a * x **(-b)
      return mu
    except: 
      print "Error Values", a, b, x

# sigmoid power law
def sigmoidpow(par, x):
	(a, b, c, d) = (abs(par[0]), abs(par[1]), abs(par[2]), 0)
	mu = a * x**(-b) * (1 - numpy.exp(-c*(x-d)))/(1 + numpy.exp(-c*(x-d)))
	return mu

# shifted power law
def shiftedpow(par, x):
	(a, b, c) = (abs(par[0]), abs(par[1]), par[2])
	mu = a * (c + x)**(-b)
	return mu

# exponential
def exponential(par, x):
	(a, b) = (abs(par[0]), abs(par[1]))
	mu = numpy.exp(a - b*x)
	return mu

def exponential_1(par, x):
  (a, b) = (abs(par[0]), abs(par[1]))
  mu = numpy.exp(a - (b + 1)*x)
  return mu


# power law + power law
def powpow(par, x):
	(a, b, c, d) = (abs(par[0]), abs(par[1]), abs(par[2]), abs(par[3]))
	mu = a * x**(-b) + c * x**(-d)
	return mu

# exponential + power law
def exppow(par, x):
	(a, b, c, d) = (abs(par[0]), abs(par[1]), abs(par[2]), abs(par[3]))
	mu = numpy.exp(a - b * x) + c * x**(-d)
	return mu
    
def swift_power(par,x):
  (alpha1, alpha2, S0, K) = (abs(par[0]), abs(par[1]), abs(par[2]), abs(par[3]))
  #mu = K*(alpha1*x**(alpha1-1) + alpha2*S0**(alpha1 - alpha2)*x**(alpha2 - 1)) / (x**alpha1 + S0**(alpha1-alpha2)*x**alpha2)**2
  mu = K*(numpy.log(10.0)*alpha1*10.0**(x*alpha1) + numpy.log(10.0)*alpha2*(S0**(alpha1 - alpha2))*10.0**(x*alpha2)) / (10.0**(x*alpha1) + (S0**(alpha1-alpha2))*10.0**(x*alpha2))**2
  return mu
  
def swift_onepower(par, x):
  (alpha1, S0, K) = (abs(par[0]), abs(par[1]), abs(par[2]))  
  mu = K*(numpy.log(10.0)*alpha1*10.0**(x*alpha1))/ (10.0**(x*alpha1) + S0**alpha1)**2  
  return mu
    
def exppowfit(x,k, hybridn):
  """
  Exponential plus power law fit
  """    
  # fit first four points to exponential distribution
  b = numpy.log(float(k[1]) / float(k[0])) / (x[1] - x[0])
  a = numpy.log(k[0]) - b * x[0]
  exppar = [a, b]
  exppar = fmin(negloglikelihood, exppar, args=(exponential, x[:4], k[:4]), maxfun=10000, maxiter=10000)
  # fit rest of data to power law
  y = k - exponential(exppar, x)
  y = y * (y > 0)
  d = -4
  #c = x[find(y>0)[-1]] ** -d
  c = x[numpy.nonzero(y>0)[-1][-1]] ** -d
  powpar = [c, d]
  try: powpar = fmin(negloglikelihood, powpar, args=(powerlaw, x[4:], y[4:]), maxfun=10000, maxiter=10000)
  except: pass
  
  # fit data to exponential + power law
  exppowparguess = numpy.hstack((exppar, powpar))
  exppowpar = fmin(hybridfit, exppowparguess, args=(exppow, x, k, hybridn), maxfun=10000, maxiter=10000)
  exppowparfit = hybridfit(exppowpar, exppow, x, k, hybridn)

  return exppowparfit, exppowpar, exppow
  

def powpowfit(x,k, hybridn):
  """
  Power plus power law fit.
  """
  # fit first four points to powerlaw distribution
  b = numpy.log(float(k[0]) / k[1]) / numpy.log(float(x[0] / x[1]))
  a = float(k[0]) / x[0]**b
  powpar = [a, b]
  powpar = fmin(negloglikelihood, powpar, args=(powerlaw, x[:4], k[:4]), maxfun=10000, maxiter=10000)

  # fit rest of data to power law
  y = k - powerlaw(powpar, x)
  y = y * (y > 0)
  d = -4
  #c = x[find(y>0)[-1]] ** -d
  c = x[numpy.nonzero(y>0)[-1][-1]] ** -d
  powpar2 = [c, d]
  powpar2 = fmin(negloglikelihood, powpar2, args=(powerlaw, x[4:], y[4:]), maxfun=10000, maxiter=10000)

  # fit data to power law + power law
  powpowparguess = numpy.hstack((powpar, powpar2))
  powpowpar = fmin(hybridfit, powpowparguess, args=(powpow, x, k, hybridn), maxfun=10000, maxiter=10000)
  powpowparfit = hybridfit(powpowpar, powpow, x, k, hybridn)
  
  return powpowparfit, powpowpar, powpow 
      
      
def shiftedpowfit(x,k):
  """
  Shifted power law fit.
  """
  # powerlaw guesses
  b = numpy.log(float(k[0]) / float(k[numpy.nonzero(k>0)[-1][-1]])) / numpy.log(x[0] / x[numpy.nonzero(k>0)[-1][-1]])
  a = k[0] / x[0] ** b
  par = [a, b, 0]

  par = fmin(fractionalsumsq, par, args=(shiftedpow, x, k), maxfun=10000, maxiter=10000)
  
  return par

def expfit(x,k):
  """
  Exponential fit.
  """
  
  # first guess
  b = numpy.log(float(k[0]) / float(k[numpy.nonzero(k>0)[-1][-1]])) / (float(x[numpy.nonzero(k>0)[-1][-1]]) - float(x[0]))
  a = numpy.log(float(k[0])) + b*x[0]
  
  par = [a,b]
  
  #par = fmin(fractionalsumsq, par, args=(exponential, x, k), maxfun=10000, maxiter=10000)
  par = fmin(negloglikelihood, par, args=(exponential, x, k), maxfun=10000, maxiter=10000)
  return par
  
  
def swift_power_fit(x,k):
  """
  Power law fit for swift background
  """           
  # first guess use numbers from the paper
  alpha1 = 1.75
  alpha2 = 0.5
  S0 = 6.5
  K = numpy.sum(k)*(x[0]**alpha1 + S0**(alpha1 - alpha2)*x[0]**alpha2)
  par = [alpha1, alpha2, S0, K]
  
  parfit = fmin(negloglikelihood, par, args=(swift_power, x, k), maxfun=10000, maxiter=10000)
  
  return parfit
  
def swift_onepower_fit(x,k):
  """
  Power law fit for swift background
  """           
  # first guess use numbers from the paper
  alpha1 = 1.75
  S0 = 6.5
  K = numpy.sum(k)*(x[0]**alpha1 + S0**alpha1)
  par = [alpha1, S0, K]
  
  parfit = fmin(negloglikelihood, par, args=(swift_onepower, x, k), maxfun=10000, maxiter=10000)
  
  return abs(parfit)
  
    
  
      
